/* Core types for the AV Engine */

#ifndef _DVE_AV_CORE_H_
#define _DVE_AV_CORE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/dve.h>
#include <dve/AVCtl/_transport.h>

#define TADDRESS_MAX_BYTES    324

enum {dveAVLocalPort=60};

typedef struct {
  u_int32 interfaceType;    /* Actually of type DveAVInterfaceType */

  u_int32 size;                       /* Size of transport-specific arg */
  char address[TADDRESS_MAX_BYTES];   /* Interface-specific data */
} DveAVNetworkAddress;

typedef struct {
  u_int32 interfaceType;     /* Actually DveAVInterfaceType */
  u_int32 transportType;     /* Actually DveAVTransportType */

  u_int32 size;                 /* Size of transport-specific arg */
  char arg[TADDRESS_MAX_BYTES]; /* Interface-specific data */

} DveAVTransportArg;

typedef struct {

  /* Information about the remote side */
  DveAVTransportArg remote;

  /* Information about the local side : set sourceAddress.type to
   * dveAVInterfaceTypeNone, and a "default" interface will be chosen to 
   * send on. This field allows you to specify a particular interface to
   * send on if more than one network interface is present on your machine.
   */
  DveAVNetworkAddress sourceAddress; 
} DveAVTransport;

typedef struct {
  u_int32 interfaceType;       /* Really DveAVInterfaceType */
  
  u_int32 size;                   /* Size of interface-specific structure */
  char iface[TADDRESS_MAX_BYTES]; /* Interface-specific data */

} DveAVNetworkIface;


typedef enum {
  dveAVSendModeNone=0,
  dveAVSendModeSend,
  dveAVSendModeReceive,
  dveAVSendModeAll
  } DveAVSendMode;

typedef enum {
  dveAVSendModeMaskSend=1<<dveAVSendModeSend,
  dveAVSendModeMaskReceive=1<<dveAVSendModeReceive
  } DveAVSendModeMask;


typedef struct {
  char name[64];               /* If this is set to a non-null string, then
				* this is used to represent the member on all
				* viewer feeds received from that member.
				*/
} DveAVMemberInfo;

typedef struct _DveAVSession *DveAVSession;

typedef enum {
  dveAVConnectionModeInvalid=0,
  dveAVConnectionModeFull,  /* Able to both send and receive */
  dveAVConnectionModeLite   /* Able to receive only */
} DveAVConnectionMode;

/* The various domains */
typedef enum {
  dveAVDomainInvalid=0,
  dveAVDomainAudio=1,
  dveAVDomainVideo=2,
  dveAVDomainTransport=3,
  dveAVDomainAudioVideo
  } DveAVDomain;

typedef enum {
  dveAVDomainMaskNone=0,
  dveAVDomainMaskAudio = 1<<dveAVDomainAudio,
  dveAVDomainMaskVideo = 1<<dveAVDomainVideo
  } DveAVDomainMask;

typedef enum {
  dveAVOK=0,
  dveAVErrorNotInited,
  dveAVErrorConnectionRefused,
  dveAVErrorIllegalTag,
  dveAVErrorIllegalDomain,
  dveAVErrorUndefined,
  dveAVErrorIllegalSize,
  dveAVErrorUnsupported,
  dveAVErrorIllegalArg,
  dveAVErrorBlock,
  dveAVErrorMemberDisabled,
  dveAVErrorInternal,
  dveAVErrorNoIncomingCall
  } DveAVError;


#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_CORE_H_ */




