/* The transport domain */


/* General notes about the transport domain:
 * The transport domain allows one to specify what mode of transport to use
 * in transmitting and receiving  data over the network. The transport 
 * capabilities are summarized below with respect to sending and receiving
 * capabilities.
 * 
 * Sending Capablities:
 * Unicast UDP : You must specify a network address to send to. You must also
 * specify a port number to send to. If the server is doing negotiation, this
 * port number will be obtained through the negotiation process, and you 
 * have no say in choosing one. If the server is not doing negotiation, you
 * must explicitly specify sending transport for a member, using 
 * dveAVSetTransport(). You can instruct the engine to bind to an arbitrary
 * IP port number by setting the port to zero in the argument to
 * dveAVSetTransport(). 
 * 
 * Multicast UDP : You must specify a group Class D address to send to. You
 * must also specify a port number, which is chosen like for unicast UDP.
 *
 * Receiving capabilities:
 * Unicast UDP : You can specify a host name and port number to bind to, or
 * just a port number, or neither. If the server is doing negotiation, you
 * may :
 * (1) Do nothing, and let the system choose a port number for you. The
 *     bind address will be set to INADDR_ANY. This is the default.
 * (2) Specify either a host address or a port number (or both) via a call to
 *     dveAVSetTransport(). (2) overrides (1).
 *
 * If the server is not doing negotiation, you must explicitly specify an 
 * address or port number or both for a member, via a call to 
 * dveAVSetTransport(). You can instruct the engine to bind to an arbitrary 
 * IP port by setting the port to zero in the arg to dveAVSetTransport(). This
 * over-rides transport info obtained via (1).
 * 
 * Multicast UDP : Like the above.
 * 
 * From the above, it should be clear that there are two paradigms: one is to
 * allow the server to choose and send to a particular port, and have the 
 * receiver bind to the same port. The other is to have the viewer choose and
 * listen on a particular port, and have the sender send to that port. C!
 * chooses the second of these paradigms when doing negotiation, hence the
 * sender is not allowed to choose a port.
 *
 * Finally, if the sending/receiving options specify selection of an 
 * arbitrary port, it is also possible to restrict the range of port numbers 
 * to bind on. The following entries in the /etc/services file will 
 * determine the min and max port numbers to use. Unfortunately, this range
 * cannot be specified on a per-connection basis.
 * 
 * opendve_video_mcast_lowwatermark        xxxxx/udp
 * opendve_video_mcast_highwatermark       yyyyy/udp
 *
 * opendve_video_unicast_lowwatermark      aaaaa/udp
 * opendve_video_unicast_highwatermark     bbbbb/udp
 *
 * For audio, substitute "audio" for "video" above.
 * 
 */

/* Below is a list of transport parameter tags. These can be used with 
 * dveAV{Get/Set}Parameter. For all parameters, the type is given in
 * comments after the name. This is the type of the data associated with this
 * tag. Most parameters are read/write : you can do either a {Get/Set}Parameter
 * on them. Read-only parameters are explicitly mentioned. It is not an error
 * to attempt to set a read-only parameter: such an attempt is ignored, and
 * nothing is done. A '*' before the type indicates that the parameter in
 * question is integer-valued : you can use the convenience functions
 * dveAV{Set/Get}IntegerParameter to set and get their values.
 */

#ifndef _DVE_AV_TRANSPORT_H_
#define _DVE_AV_TRANSPORT_H_

#ifdef __cplusplus
extern "C"
#endif

#include <dve/AVCtl/AVCtl.h>

typedef enum {
  dveAVTransportTagInvalid=0,

  /* Type : DveAVTransportArgAvailInterfaces; List all avail interfaces; This
   * parameter is read-only 
   */
  dveAVTransportTagAvailInterfaces

  } DveAVTransportTag;


/* Arg(s) for dveAVTransportTagAvailInterfaces */

typedef struct DveAVNetworkIface DveAVTransportArgCurrentInterface;

typedef struct {
  DveAVNetworkIface iface;    /* Interface info */
  u_int32 mask;               /* A mask of available transport types for this
			       * interface
			       */
} DveAVTransportArgInterface;

typedef struct {
  u_int32 numIfaces;
  DveAVTransportArgInterface *ifaces;
} DveAVTransportArgAvailInterfaces;


/* General arg for an integer-valued transport parameter */
typedef struct {
  u_int32 valid; /* "valid" tells us whether the value is valid */
  u_int32 value;
} DveAVTransportParameter;


#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_TRANSPORT_H_ */
