/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the protocol for packet filtering/access control.
 *
 * Requires:
 *
 * Includes:
 *  <dve/packetFilteringLow.h>
 *
 * Implemented by:
 *  See <dve/packetFiltering.h>.
 *
 * Classes:
 *
 * Types:
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 16 March 1995
 *
 * Known Bugs:
 *
 * Notes:
 *  This is the very simple filtering system, which blocks all non-local
 *   packets and allows all local ones through.
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_PACKET_FILTERING_PROTOCOL_H_
#define _DVE_PACKET_FILTERING_PROTOCOL_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/packetFilteringLow.h>

enum {dvePacketFilteringProtocol=91,
	dvePacketFilterPort=dvePacketFilteringProtocol,
	dvePacketFilteringProtVers=1
      };

typedef enum {
  dvePacketFilteringTagInvalid=0,
  dvePacketFilteringTagQuery,
  dvePacketFilteringTagReply,
  dvePacketFilteringTagSet,
  dvePacketFilteringTagSetDefault,
  dvePacketFilteringTagNotify,
  dvePacketFilteringTagNotification,
  dvePacketFilteringTagAck
  } DvePacketFilteringTag;

typedef struct {
  DvePort port; /* 0 for default state */
} DvePacketFilteringMsgArgQuery;

typedef struct {
  DvePacketFilteringState blocked;
} DvePacketFilteringMsgArgReply;

typedef struct {
  DvePort port;
  DvePacketFilteringState blocked;
} DvePacketFilteringMsgArgSet;

typedef struct {
  DvePacketFilteringState blocked;
} DvePacketFilteringMsgArgSetDefault;

typedef struct {
  DvePort port; /* 0 for "all plugins" */
  Bool16 notify;
} DvePacketFilteringMsgArgNotify;

typedef struct {
  DvePort port; /* 0 for default state--any plugin getting any notification
		 *  will be notified of the default state.
		 */
  DvePacketFilteringState blocked;
} DvePacketFilteringMsgArgNotification;

typedef struct {
  Bool16 success;
} DvePacketFilteringMsgArgAck;

typedef union {
  DvePacketFilteringMsgArgQuery query;
  DvePacketFilteringMsgArgReply reply;
  DvePacketFilteringMsgArgSet set;
  DvePacketFilteringMsgArgSetDefault setDefault;
  DvePacketFilteringMsgArgNotify notify;
  DvePacketFilteringMsgArgNotification notification;
  DvePacketFilteringMsgArgAck ack;
} DvePacketFilteringMsgArg;

typedef struct {
  u_int32 protVers;
  u_int32 reqID; /* Arbitrary; assigned by sender, except that, for a Reply,
		  * must be same as for originating Query; for an Ack or Nak,
		  * must be same as for packet being acknowledged or refused.
		  */
  DvePacketFilteringMsgArg arg;
} DvePacketFilteringMsg;

/*
 * Implementation description:
 *  The packet filtering manager (PFM), which is built into the engine,
 *   listens for the packet filtering protocol, to provide services to the
 *   plugins.  The packet router asks the PFM for a go/no-go on each packet
 *   addressed to a local plugin; the PFM checks its data on that plugin
 *   and decides.
 *
 *  The PFM keeps a state for each plugin whose state has been set explicitly.
 *   There is also a default state, which can be set only by the plugin
 *   manager.  If the plugin's state has not been set, or if it is
 *   dvePacketFilteringStateDefault, it is treated as being equal to the
 *   default state.
 *
 *  The PFM reacts to packets as follows:
 *   Query: the PFM looks up the given plugin's state and sends a Reply
 *    packet to the sender of the Query, with arg.reply.blocked set to
 *    the plugin's state and arg.reply.reqID the same as that of the Query.
 *   Reply: the PFM ignores Reply.
 *   Set: if the sender is the given plugin or the plugin manager, the
 *    PFM looks up the given plugin's state and sets it to arg.set.blocked;
 *    if this is a change, it will send a Notification to each plugin which
 *    has submitted a Notify for the given plugin.  If the sender is neither
 *    of the two legal plugins, the PFM will send it a negative Ack; otherwise,
 *    it will send it a positive Ack.
 *   SetDefault: if the sender is the plugin manager, the PFM sets the default
 *    state; if this is a change, it sends Notifications to each plugin which
 *    has submitted a Notify for any plugin which is using the default state.
 *    If the sender is not the plugin manager, the PFM will send it a negative
 *    Ack; otherwise, it will send it a positive Ack.
 *   Notify: if arg.notify.notify is true, the PFM adds the sender to the
 *    list of plugins requesting notification for the given plugin; otherwise,
 *    the sender is removed from this list.
 *   Notification: the PFM ignores Notification.
 *   Ack: the PFM ignores Ack.
 *
 *  Plugins should react to packets as follows:
 *   Query, Set, SetDefault, Notify: ignored.
 *   Reply: the reply should be delivered to whatever entity issued the
 *    query with the same name.  In libdvePacketFiltering, this involves
 *    exiting the blocking event loop inside of
 *    dveAccessControlIsBlockingRemote().
 *   Notification: the notification should be delivered to whatever entity
 *    or entities have requested it.  In libdvePacketFiltering, this involves
 *    invoking the appropriate callback(s); in addition, the state of the
 *    given plugin is cached (see implementation note below).
 *   Ack: the plugin may note, somehow, that the request has succeeded or
 *    failed.  This could be used to implement a blocking form of
 *    libdvePacketFiltering's dveAccessControlSetBlockingRemote().
 *
 *  Implementation note: in libdvePacketFiltering,
 *   dveAccessControlIsBlockingRemote() does not always send a Query.  If
 *   we have the given plugin's state cached (due to a Notification), we
 *   will simply return it.  Otherwise, we send a Query and a Notify, then
 *   wait for the Reply.
 */

#ifdef __cplusplus
}
#endif

#endif /*_DVE_PACKET_FILTERING_PROTOCOL_H_*/
