#ifndef _DVE_CLIPBOARD_API_H_
#define _DVE_CLIPBOARD_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/clipboardLow.h>

typedef struct _DveClipboard* DveClipboard;

DveClipboard dveClipOpen(DveSession session,
			 DveClipboardName name,
			 DveUserHandle owner);
void dveClipClose(DveClipboard clipboard);

int dveClipLock(DveClipboard clipboard);
int dveClipUnlock(DveClipboard clipboard);

int dveClipWipe(DveClipboard clipboard);
int dveClipPut(DveClipboard clipboard,
	       DveClipItem* item);
DveClipItem* dveClipGet(DveClipboard clipboard,
			DveClipFormatName format);

Bool16 dveClipIsFormatPresent(DveClipboard clipboard,
			      DveClipFormatName format);

typedef void (*DveClipNotifyCallback)(DveClipboard clipboard);
int dveClipNotify(DveClipboard clipboard,
		  DveClipNotifyCallback callback);
int dveClipDontNotify(DveClipboard clipboard);

typedef void (*DveClipFmtNotifyCallback)(DveClipboard clipboard,
					 DveClipFormatName format,
					 Bool16 present);
int dveClipNotifyFormat(DveClipboard clipboard,
			DveClipFormatName format,
			DveClipFmtNotifyCallback callback);
int dveClipDontNotifyFormat(DveClipboard clipboard,
			    DveClipFormatName format);

typedef void (*DveClipRenderCallback)(DveClipRenderArg* renderArg);
int dveClipRegisterRenderer(DveSession session,
			    u_int32 numFormats,
			    DveClipFormatName* formats);
int dveClipUnregisterRenderer(DveSession session);

DveSession dveClipSession(DveClipboard clipboard);
DveUserHandle dveClipOwner(DveClipboard clipboard);
const char* dveClipName(DveClipboard clipboard);

Bool16 dveClipBlocked(void);

/* A mechanism to get notified when the clipboard becomes unblocked.  The
 *  callback will be invoked from within a work proc, added after all
 *  clipboard notifications have been handled.  The callback will be
 *  invoked only once; after that, the watcher will be removed.
 */

typedef void (*DveClipUnblockedCallback)(void* context);

typedef struct _DveClipUnblockedWatcher* DveClipUnblockedWatcher;

DveClipUnblockedWatcher dveClipWatchBlockage(DveClipUnblockedCallback callback,
					     void* context);

void dveClipDontWatchBlockage(DveClipUnblockedWatcher watcher);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_CLIPBOARD_API_H_*/

