/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the API for Conference Manager functions (both ordinary OpenDVE
 *   requests & the Conference Manager protocol).
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Implemented by:
 *  libdve{Xt,Xview}.a (specifically, dve.c).
 *  libdveConfMgr.a
 *
 * Types:
 *
 * Functions:
 *
 * Author:
 *  francis, 8 June 1994
 *
 * Known Bugs:
 *
 * Notes:
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_CONFERENCE_MANAGER_API_H_
#define _DVE_CONFERENCE_MANAGER_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/_confMgr.h>

/*
 * An invitation callback is invoked when an invitation to a conference
 * arrives.
 */
typedef void (*DveConfMgrInvitationCallback)(DveSession dveSession,
					     DveConfMgrInvitedArg* arg);
typedef void (*DveConfMgrBadInvitationCallback)(DveSession dveSession,
						DveConfMgrBadInvitedArg* arg);

int dveConfMgrSetInvitationCallback(DveSession dveSession,
				    DveConfMgrInvitationCallback callback);
int dveConfMgrSetBadInvitationCallback(DveSession dveSession,
				       DveConfMgrBadInvitationCallback callback);
int dveConfMgrSetUserCallback(DveSession dveSession,
			      DveUserCallback callback);

void dveConfMgrAcceptInvitation(DveSession dveSession);
void dveConfMgrRejectInvitation(DveSession dveSession);

typedef struct {
  const char* userName;
  const char* hostName; /* Can be an IP # */
} DveConfMgrUserAddrOld;

typedef struct {
  const char* userName;
  const char* hostName; /* Can be an IP # */
  DveIPAList* addrs; /* Can be NULL */
} DveConfMgrUserAddr;

int dveConfMgrInitiateConferenceOld(DveSession dveSession,
				    DveConfMgrUserAddrOld* users,
				    int numUsers);
int dveConfMgrAddMemberOld(DveSession dveSession, DveConfMgrUserAddrOld* user);
int dveConfMgrInitiateConference(DveSession dveSession,
				 DveConfMgrUserAddr* users,
				 int numUsers);
int dveConfMgrAddMember(DveSession dveSession, DveConfMgrUserAddr* user);
int dveConfMgrLeaveConference(DveSession dveSession);

/* Set the Do Not Disturb flag */
/*int dveSetDND(DveSession dveSession, Bool16 DND);*/
int dveLeaveOffice(DveSession dveSession);
int dveReturnToOffice(DveSession dveSession);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_CONFERENCE_MANAGER_API_H_*/
