/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares a convenience library for the calling engine to communicate
 *   with the conference engine.  This serves as the external spec for the
 *   conference engine/calling engine interface (since InSoft will write the
 *   convenience library).
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Includes:
 *  <dve/requests.h>
 *
 * Implemented by:
 *  lib/dveConfToCalling
 *
 * Types:
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 16 May 1995
 *
 * Known Bugs:
 *
 * Notes:
 *  For descriptions of DveSessions and DveEvents, see the OpenDVE manual.
 *
 * Change Log:
 *  $Log$
 */
#ifndef _OPENDVE_CONFERENCE_ENGINE_TO_COMMWARE_ENGINE_API_H_
#define _OPENDVE_CONFERENCE_ENGINE_TO_COMMWARE_ENGINE_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/requests.h>

/* Unless otherwise noted, the return value for a function returning
 *  int is 0 on success or <0 on error.  On error, consult dveError
 *  (see <dve/error.h>) for the error.
 */

/* Request callback, to be invoked when a proxy OpenDVE request arrives */
typedef void (*DveC2CRequestCallback)(DveSession dveSession,
				      DveRequest* request);

/* Set the request callback.  Call this even if you're just setting it
 *  to NULL; that'll initialize the library.
 */
int dveC2CSetReqCallback(DveSession dveSession,
			 DveC2CRequestCallback callback);

/* Address callback, to be invoked when the engine reports its addresses. */
typedef void
  (*DveC2CAddrCallback)(DveSession dveSession,
			const DveIPAList* addrs,
			u_int32 switchID);

/* Set the address callback.  If the address has already been reported, the
 *  callback will be invoked immediately (i.e., from within this function).
 */
int dveC2CSetAddrCallback(DveSession dveSession,
			  DveC2CAddrCallback callback);

/* Get the addresses reported by the engine.  Returns -1 on error (most
 *  likely to be dveErrDataUnavail, meaning the engine hasn't reported
 *  yet), 0 on success.
 */
int dveC2CGetMyAddr(DveSession dveSession,
		    DveIPAList* addrs,
		    u_int32* switchID);

/* Send an event to the engine, to let it track the state of the
 *  conference.
 */
int dveC2CSendEvent(DveSession dveSession,
		    DveEvent* event);

/* Note: the following four functions replace the one
 *  dveC2CReportOpenDVEAddr() from the original proposal.
 */
/* Tell the engine the address to connect to for the given member.  If
 *  addrs is NULL, it means this member is on the other side of the MMCS,
 *  and the engine should use the MMCS address (see below).
 */
int dveC2CReportOpenDVEAddr(DveSession dveSession,
			    DveUserHandle member,
			    const DveIPAList* addrs,
			    u_int32 switchID);

/* Tell the engine the address for the cell switch in the MMCS. */
int dveC2CReportMMCSAddr(DveSession dveSession,
			 const DveIPAList* addrs,
			 u_int32 switchID);

/* Tell the engine it can open/close connections for OpenDVE data.  Engine
 *  may, or may not, defer opening until a plugin wants to send data.
 */
int dveC2COpenDataConns(DveSession dveSession);
int dveC2CCloseDataConns(DveSession dveSession);

#ifdef __cplusplus
}
#endif

#endif /*_OPENDVE_CONFERENCE_ENGINE_TO_COMMWARE_ENGINE_API_H_*/
