/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares error codes for OpenDVE, as well as the global variable dveError
 *   and a function to interpret an OpenDVE error message to a string.
 *
 * Implemented by:
 *  dve.c
 *
 * Types:
 *  DveError
 *
 * Functions:
 *  dveErrorString()
 *
 * Variables:
 *  dveError
 *
 * Author:
 *  francis, ?? Feb 1994
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_ERRORS_H_
#define _DVE_ERRORS_H_

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
  dveErrOK=0,                     /* no error */
  dveErrBadPtr,                   /* Invalid pointer */
  dveErrBadProtocol,              /* Bad protocol # */
  dveErrBadPort,                  /* Bad port # */
  dveErrAlreadyConnected,         /* Session is already connected */
  dveErrNotConnected,             /* Session is not connected */
  dveErrOutOfMemory,              /* Out of memory */
  dveErrBadFifoPath,              /* Invalid path used for engine FIFO */
  dveErrUnknown,                  /* Unknown error */
  dveErrBadEventTag,              /* Bad event tag */
  dveErrDataUnavail,              /* Requested data is unavailable */
  dveErrUnimplemented,            /* Functionality is not implemented */
  dveErrBadArg,                   /* Invalid argument */
  dveErrAlreadyRegistered,        /* Session is already registered with WM */
  dveErrNotRegistered,            /* Session is not registered with WM */
  dveErrCantRegister,             /* Registration failed */
  dveErrArrayTooSmall,            /* Too many elements to fit in array */
  dveErrOutOfFDs,                 /* Out of file descriptors */
  dveErrOutOfOther,               /* Out of some other resource */
  dveErrNoPerms,                  /* Access denied */
  dveErrNotOpen,                  /* File (?) not open */
  dveErrUseErrno,                 /* Check errno for the error */
  dveErrConnRefused,              /* Connection refused */
  dveErrNetUnreachable,           /* Network unreachable */
  dveErrNoRecipients,             /* Null recipient list specified */
  dveErrNotFound,                 /* Null recipient list specified */
  dveErrInUse,                    /* Port or other resource in use */
  dveErrBadState                  /* Wrong state for operation (vague) */
  } DveError;

extern DveError dveError;
const char* dveErrorString(DveError dveError);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_ERRORS_H_*/
