/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares functions to add file descriptors to watch for I/O.
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Includes:
 *
 * Implemented by:
 *  OpenDVE/dveXt.c, dveXView.c, dveIO.c, dve.c
 *
 * Types:
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 26 Apr. 1995
 *
 * Known Bugs:
 *
 * Notes:
 *
 * Change Log:
 *  $Log$
 */
#ifndef _OPENDVE_INPUT_OUTPUT_WATCHERS_H_
#define _OPENDVE_INPUT_OUTPUT_WATCHERS_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

/* Thunk for a file descriptor being watched */
typedef struct _DveIOWatcher* DveIOWatcher;

/* Ways to watch a file descriptor */
typedef enum {
  dveIOWatcherTypeInvalid=0,
  dveIOWatcherTypeRead,
  dveIOWatcherTypeWrite,
  dveIOWatcherTypeExcept
  } DveIOWatcherType;

/* Callback invoked when there's activity on a file descriptor */
typedef void (*DveIOWatcherCallback)(DveIOWatcher watcher,
				     void* context);

/* Start watching a file descriptor.  It is an error (dveBadArg) for
 *  callback to be null.  context will be passed into the callback
 *  when invoked.
 *
 * Warning: at this time, having more than one watcher on the same fd
 *  is not supported; you can add a second one, but things may get
 *  corrupted when you delete one of them.
 */
DveIOWatcher dveIOWatch(DveSession dveSession,
			int fd,
			DveIOWatcherType type,
			DveIOWatcherCallback callback,
			void* context);
/* Stop watching the file descriptor; delete the watcher */
void dveIOWatcherClose(DveIOWatcher watcher);

/* Stop watching, but don't delete the watcher (used when you expect to
 *  have to start watching again later).
 */
int dveIOWatcherSuspend(DveIOWatcher watcher);
/* Resume (start watching again) after a suspend. */
int dveIOWatcherResume(DveIOWatcher watcher);

/* Get info about the watcher. */
DveSession dveIOWatcherGetSession(DveIOWatcher watcher);
int dveIOWatcherGetFd(DveIOWatcher watcher);
DveIOWatcherType dveIOWatcherGetType(DveIOWatcher watcher);
void* dveIOWatcherGetContext(DveIOWatcher watcher);

#ifdef __cplusplus
}
#endif

#endif /*_OPENDVE_INPUT_OUTPUT_WATCHERS_H_xb*/
