/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the OpenDVE memory management system (which is really a
 *   shell to let plugins use whatever memory manager they want).
 *
 * Includes:
 *  <dve/malloc.h>
 *
 * Implemented by:
 *  dve.c
 *
 * Types:
 *  dveMemoryAllocator
 *  dveMemoryFreer
 *  DveMemoryMgr
 *
 * Functions:
 *  dveInstallMemoryMgr
 *  dveMalloc
 *  dveFree
 *
 * Author:
 *  francis, ?? Mar 1994
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_MEMORY_SUBSYSTEM_H_
#define _DVE_MEMORY_SUBSYSTEM_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/malloc.h>

/* Pointers to functions to allocate/free memory */
typedef IS_mallocReturnType* (*dveMemoryAllocator)(IS_mallocSizeType size);
typedef void (*dveMemoryFreer)(IS_mallocReturnType* p);

/* Abstraction of a memory manager */
typedef struct {
  dveMemoryAllocator allocator;
  dveMemoryFreer freer;
} DveMemoryMgr;

/* Set the current memory manager */
void dveInstallMemoryMgr(DveMemoryMgr* mgr);
/* Allocate a block of memory, using the current memory manager */
void* dveMalloc(IS_mallocSizeType size);
/* Free a block of memory, using the memory manager that allocated it. */
void dveFree(void* p);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_MEMORY_SUBSYSTEM_H_*/
