/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the API for controlling packet filtering (a new capability
 *   to be added to the engine for GBCS).
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Includes:
 *  <dve/packetFilteringLow.h>
 *
 * Implemented by:
 *  lib/dvePacketFiltering
 *  Something in the engine.
 *
 * Types:
 *  DveAccessControlWatcher
 *  DveAccessControlCallback
 *
 * Functions:
 *  dveAccessControlSetBlocking()
 *  dveAccessControlIsBlocking()
 *  dveAccessControlDisable()
 *  dveAccessControlEnable()
 *  dveAccessControlSetBlockingRemote()
 *  dveAccessControlIsBlockingRemote()
 *  dveAccessControlAddWatcher()
 *  dveAccessControlDelWatcher()
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 16 March 1995
 *
 * Known Bugs:
 *
 * Notes:
 *  This is the very simple filtering system, which blocks all non-local
 *   packets and allows all local ones through.
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_PACKET_FILTERING_API_H_
#define _DVE_PACKET_FILTERING_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/packetFilteringLow.h>

/* Unless otherwise noted, the return value for a function returning
 *  int is 0 on success or <0 on error.  A function returning
 *  DvePacketFilteringState returns dvePacketFilteringStateInvalid on
 *  error.  On error, consult dveError (see <dve/error.h>) for the error.
 */

int dveAccessControlSetBlocking(DveSession dveSession,
				DvePacketFilteringState state);
DvePacketFilteringState dveAccessControlIsBlocking(DveSession dveSession);

/* Convenience wrappers around dveAccessControlSetBlocking(), above */
int dveAccessControlDisable(DveSession dveSession);
int dveAccessControlEnable(DveSession dveSession);

/* These two are available only to the plugin manager, the only plugin allowed
 *  to control another plugin's blocking.
 */
int dveAccessControlSetBlockingRemote(DveSession dveSession,
				      DvePort plugin,
				      DvePacketFilteringState state);
int dveAccessControlSetDefaultBlocking(DveSession dveSession,
				       DvePacketFilteringState state);

/* This is sometimes a blocking call (i.e., it includes an OpenDVE event
 *  loop).  The "sometimes" is because the watcher system (see below) is
 *  used to let us cache the state of plugins we're interested in, so we
 *  don't have to ask every time.
 */
DvePacketFilteringState dveAccessControlIsBlockingRemote(DveSession dveSession,
							 DvePort plugin);

/* Get the default state.  As with dveAccessControlIsBlockingRemote(),
 *  above, this is sometimes a blocking call.  Note, though, that having
 *  *any* watcher installed will let us be notified when the default
 *  blocking is changed, so that this call can be very cheap to use
 *  when we get notified that somebody's state is now the default.
 */
DvePacketFilteringState
  dveAccessControlGetDefaultBlocking(DveSession dveSession);

/* The watcher system is a mechanism to track a plugin's blocking state.
 *  This would be useful for the plugin manager, which needs to be able
 *  to show the state of all managed plugins, and so needs to be notified
 *  when a plugin changes its own state.
 */

/* Thunk for the eye we're keeping on a plugin's blocking state */
typedef struct _DveAccessControlWatcher* DveAccessControlWatcher;

/* Callback to invoke when a DveAccessControlWatcher is triggered. */
typedef void (*DveAccessControlCallback)(DveSession dveSession,
					 DvePort plugin,
					 DvePacketFilteringState state,
					 void* context);

/* Create a watcher for the given plugin.  context will be passed to
 *  the watcher when the callback is invoked.  Returns NULL on error.
 */
DveAccessControlWatcher dveAccessControlAddWatcher(DveSession dveSession,
						   DvePort plugin,
						   DveAccessControlCallback callback,
						   void* context);

/* Delete the given watcher (stop watching). */
void dveAccessControlDelWatcher(DveAccessControlWatcher watcher);

const char* dvePacketFilteringStateName(DvePacketFilteringState state);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_PACKET_FILTERING_API_H_*/
