#ifndef _DVE_PLUGIN_MANAGEMENT_H_
#define _DVE_PLUGIN_MANAGEMENT_H_

#ifdef __cplusplus
extern "C" {
#endif

enum {dvePluginManagementProtocol=5, dvePluginManagementProtVers=0};

typedef enum {
  dvePluginManagementMsgTagInvalid=0,
  dvePluginManagementMsgTagSysSpecificFirst,
  dvePluginManagementMsgTagRegister,   /* [Un]register as someone who wants */
  dvePluginManagementMsgTagUnregister, /*  this plugin to keep running.  See
					*  below.
					*/
  dvePluginManagementMsgTagRegisterAck, /* Acknowledge Register */
  dvePluginManagementMsgTagRegisterNak, /* Register failed */

  dvePluginManagementMsgTagSysSpecificLast=
    dvePluginManagementMsgTagSysSpecificFirst+1000,

  dvePluginManagementMsgTagShow,
  dvePluginManagementMsgTagHide,
  dvePluginManagementMsgTagQuit, /* But see below */
  dvePluginManagementMsgTagReady /* Sent from plugin to confmgr when the
				  *  plugin's initialized & ready to use.
				  */
  } DvePluginManagementMsgTag;

#ifdef __cplusplus
}
#endif

/* Register, Unregister, Quit interact as follows:
 *
 *  * A plugin which does not support registration should ignore Register
 *    and Unregister altogether, and just quit when it gets a Quit message.
 *    Further directions here assume the plugin does support registration.
 *
 *  * The plugin must keep a roster of registered clients, and must keep
 *     a plugin connection open to each one.
 *
 *  * When the plugin receives a Register, the plugin must reply with
 *    RegisterAck.
 *
 *  * The plugin must keep a "I should quit" flag (initially false).
 *
 *  * When a client sends an Unregister, or when the plugin connection
 *     dies, the plugin must remove that client from the roster.
 *
 *  * When the plugin receives a Quit, it must set the "I should quit" flag
 *    to true.
 *
 *  * If the roster is empty, and the "I should quit" flag is true, the
 *    plugin should exit.
 *
 *  * Clients should open plugin connections, to be notified when the
 *    plugin they're interested in exits (in case it dies prematurely,
 *    or something).  However, this is not part of this protocol, but
 *    simply an explanation of why this protocol does not address such
 *    notification.
 *
 * This scheme allows the INTV help viewer to work semi-smoothly with C!
 *  4.0.1: if the help viewer is spawned by C!, and then INTV is run, the
 *  viewer will not exit when C! exits, because INTV will be registered.  If
 *  it's started by INTV, on the other hand, it will exit when INTV does.
 *  This is unavoidable.  However, C! 4.1 will do registration, and will
 *  then work smoothly with INTV.  (Note that the 4.0.1 interworking
 *  scheme requires that C! run with the 4.1 viewer and 4.0.2 (?) engine
 *  shipped with INTV.)
 */

#endif /*_DVE_PLUGIN_MANAGEMENT_H_*/
