#ifndef _OPEN_DVE_USER_INFO_LIB_H_
#define _OPEN_DVE_USER_INFO_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  dveUserInfoTagInvalid=0,
  dveUserInfoTagName,
  dveUserInfoTagTitle,
  dveUserInfoTagCompany,
  dveUserInfoTagAddress1,
  dveUserInfoTagAddress2,
  dveUserInfoTagAddress3,
  dveUserInfoTagPhone,
  dveUserInfoTagFax,
  dveUserInfoTagMisc,
  dveUserInfoTagNickName,
  dveUserInfoTagIcon
} DveUserInfoTag;

/* Used only by the confmgr */
int dveUserInfoSet(DveSession dveSession,
		   DveUserHandle handle,
		   DveUserInfoTag tag,
		   u_int32 datalen,
		   const char* data);

/* Wrappers, also used only by the confmgr */
int dveUserInfoSetStr(DveSession dveSession,
		      DveUserHandle handle,
		      DveUserInfoTag tag,
		      const char* data);
int dveUserInfoSetIcon(DveSession dveSession,
		       DveUserHandle handle,
		       u_int32 datalen,
		       const char* data /* Icon in DIB format. */
		       );

/* Returns data on success, NULL on failure.  *datalen is filled out
 *  to the amount of data (including the 0, if it's a string).  If
 *  datalen is NULL, it won't be touched.
 */
const char* dveUserInfoGet(DveSession dveSession,
			   DveUserHandle handle,
			   DveUserInfoTag tag,
			   u_int32* datalen);

/* Wrapper that just calls dveUserInfoGet() with datalen NULL.  (Also
 *  checks the tag to make sure it's not the icon.)
 */
const char* dveUserInfoGetStr(DveSession dveSession,
			      DveUserHandle handle,
			      DveUserInfoTag tag);

/* Wrapper around dveUserInfoGetStr(), with tag dveUserInfoTagNickName. */
const char* dveUserInfoGetNick(DveSession dveSession,
			       DveUserHandle handle);

/* Get the nickname, or the user@host name if the nickname is unavailable */
const char* dveUserInfoGetNickOrAddr(DveSession dveSession,
				     DveUserHandle handle);

/* Wrapper that calls dveUserInfoGet() with the icon tag.  Returns the
 *  icon in DIB format.
 */
const char* dveUserInfoGetIcon(DveSession dveSession,
			       DveUserHandle handle,
			       u_int32* datalen);

/* Function to invoke when a tag changes. */
typedef void (*DveUserInfoWatcherCallback)(DveSession dveSession,
					   DveUserHandle handle,
					   DveUserInfoTag tag,
					   const char* data,
					   u_int32 datalen,
					   void* context);

typedef struct _DveUserInfoWatcher* DveUserInfoWatcher;

/* Add a watcher.  context will be passed along to the callback. */
DveUserInfoWatcher dveUserInfoAddWatcher(DveSession dveSession,
					 DveUserHandle handle,
					 DveUserInfoTag tag,
					 DveUserInfoWatcherCallback callback,
					 void* context);
void dveUserInfoDelWatcher(DveUserInfoWatcher watcher);

#ifdef __cplusplus
}
#endif

#endif /*_OPEN_DVE_USER_INFO_LIB_H_*/
