#include <dve/AVCtl/AVCtl.h>
#include <dve/AVCtl/video.h>

typedef struct _menu Menu;
typedef struct _menuItem MenuItem;

/* Return False to exit the menu, True to continue with the menu */
typedef int (*menuHandler) (DveAVSession session);  


struct _menuItem {
  char *message;
  int (*handler) (DveAVSession session); /* Ignored if subMenu */

  Menu *subMenu;           /* 0 if no Sub-menu */
};

struct _menu {
  char *name;              /* Name of menu */
  int (*initial) (DveAVSession);
  void (*final) (DveAVSession);
  MenuItem *items;
};

void execMenu(Menu *menu, DveAVSession session);

#define MAXMEMBERS 5

int addMember(DveAVSession session);

int removeMember(DveAVSession session);  
int start(DveAVSession session);
int stop(DveAVSession session);
int disableVideo(DveAVSession session);
int enableVideo(DveAVSession session);
int disableAudio(DveAVSession session);
int enableAudio(DveAVSession session);
void init();
int startViewer(DveAVSession session);

int listInterfaces(DveAVSession session);

int initial(DveAVSession session);
void final(DveAVSession session);
int modifyInitial(DveAVSession session);

int enableMember(DveAVSession session);
int disableMember(DveAVSession session);
int setSendingTransport(DveAVSession session);
int setReceivingTransport(DveAVSession session);
int quit(DveAVSession session);

int startVideoMonitor(DveAVSession session), stopVideoMonitor(DveAVSession);
int changeVideoBoard(DveAVSession session), 
  changeVideoCompression(DveAVSession session);
int boardParamsInitial(DveAVSession session);
int exitVideoMenu(DveAVSession);
int exitParamsMenu(DveAVSession);
int pause(DveAVSession), resume(DveAVSession);

int setParam(DveAVSession), getParam(DveAVSession);
int show(DveAVSession), hide(DveAVSession);

int setInputPort(DveAVSession);
int setOutputPort(DveAVSession);
int setPlayVol(DveAVSession);
int setSendVol(DveAVSession);
int setSilence(DveAVSession);
int setEcho(DveAVSession);
int setsync(DveAVSession);

int exitAudioMenu(DveAVSession);

int showAudioInterface(DveAVSession);
int hideAudioInterface(DveAVSession);

int loadVideoPrefs(DveAVSession);
int saveVideoPrefs(DveAVSession);
int loadVideoDefaults(DveAVSession);

int loadAudioPrefs(DveAVSession);
int saveAudioPrefs(DveAVSession);
int loadAudioDefaults(DveAVSession);

int playAudioFile(DveAVSession session);
int genDTMFTone(DveAVSession session);

int stopAudioFile(DveAVSession session);
int stopDTMFTone(DveAVSession session);

#ifdef _MAIN_PROG_

MenuItem mainMenuItems[], audioMenuItems[], videoMenuItems[], 
  modifyMenuItems[], boardParamsMenuItems[], setAudioParamsMenuItems[],
  transportMenuItems[], audioTonesItems[];


Menu mainMenu = {
  "Main Menu", 
  initial, 
  final, 
  mainMenuItems
  };
  
Menu audioMenu = {
  "Audio Menu",
  initial,
  final,
  audioMenuItems
  };

Menu transportMenu = {
  "Transport Menu",
  initial,
  final,
  transportMenuItems
  };

Menu videoMenu = {
  "Video Menu",
  initial,
  final,
  videoMenuItems
  };

Menu boardParamsMenu = {
  "Set/Get Board parameters",
  boardParamsInitial,
  final,
  boardParamsMenuItems
  };

Menu tonesMenu = {
  "Audio Tones Menu",
  initial,
  final,
  audioTonesItems
  };

char modifyMemberString[128];
Menu modifyMemberMenu = {
  modifyMemberString,
  modifyInitial,
  final,
  modifyMenuItems
  };

Menu setAudioParamsMenu = {
  "Set an Audio Parameter",
  initial,
  final,
  setAudioParamsMenuItems
  };

MenuItem mainMenuItems[] = {
  {"Add a Member", addMember, 0},
  {"Remove a Member", removeMember, 0},
  {"Modify Member Information", 0, &modifyMemberMenu},
  {"Enable Audio", enableAudio, 0},
  {"Disable Audio", disableAudio, 0},
  {"Enable Video", enableVideo, 0},
  {"Disable Video", disableVideo, 0},
  {"Start Sending", start, 0},
  {"Stop Sending", stop, 0},
  {"Pause", pause, 0},
  {"Resume", resume, 0},
  {"Video Sub-menu", 0, &videoMenu},
  {"Audio Sub-menu", 0, &audioMenu},
  {"Transport Sub-menu", 0, &transportMenu},
  {"Audio Tones Sub-Menu", 0, &tonesMenu}, 
  {"Quit", quit, 0},
  {0, 0, 0}
};

MenuItem audioMenuItems[] = {
  {"Set the value of an Audio Parameter", 0, &setAudioParamsMenu},
  {"Show audio interface", showAudioInterface, 0},
  {"Hide audio interface", hideAudioInterface, 0},
  {"Load Preferences", loadAudioPrefs, 0},
  {"Save Preferences", saveAudioPrefs, 0},
  {"Load Defaults", loadAudioDefaults, 0},
  {"Exit this menu", exitAudioMenu, 0},
  {0, 0, 0}
};

MenuItem audioTonesItems[] = {
  {"Play an audio file", playAudioFile, 0},
  {"Generate a DTMF tone", genDTMFTone, 0},
  {"Stop playing an audio file", stopAudioFile, 0},
  {"Stop playing a DTMF tone", stopDTMFTone, 0},
  {0, 0, 0}
};

MenuItem videoMenuItems[] = {
  {"Start Self-View", startVideoMonitor, 0},
  {"Stop Self-View", stopVideoMonitor, 0},
  {"Change Video Board", changeVideoBoard, 0},
  {"Change Current Compression", changeVideoCompression, 0},
  {"Other Board Parameters", 0, &boardParamsMenu},
  {"Show Board Controls", show, 0},
  {"Hide Board Controls", hide, 0},
  {"Load preferences", loadVideoPrefs, 0},
  {"Save preferences", saveVideoPrefs, 0},
  {"Load defaults", loadVideoDefaults, 0},
  {"Exit this menu", exitVideoMenu, 0},
  {0, 0, 0}
};

MenuItem transportMenuItems[] = {
  {"List available network interfaces", listInterfaces, 0},
  {0, 0, 0}
};

MenuItem modifyMenuItems[] = {
  {"Enable Audio/Video", enableMember, 0},
  {"Disable Audio/Video", disableMember, 0},
  {"Set Sending transport", setSendingTransport, 0},
  {"Set Receiving Transport", setReceivingTransport, 0},
  {0, 0, 0}
};

MenuItem boardParamsMenuItems[] = {
  {"Get the value of a board Param", getParam, 0},
  {"Set the Value of a board param", setParam, 0},
  {"Exit this menu", exitParamsMenu, 0},
  {0, 0, 0}
};

MenuItem setAudioParamsMenuItems[] = {
  {"Set Input port", setInputPort, 0},
  {"Set Output port", setOutputPort, 0},
  {"Set Play Vol", setPlayVol, 0},
  {"Set Sending Vol", setSendVol, 0},
  {"Set Silence Sensor", setSilence, 0},
  {"Set Echo Cancellation", setEcho, 0},
  {"Set AV Sync Value", setsync, 0},
  {0, 0, 0}
};


#endif /* _MAIN_PROG_ */




