#include <stdio.h>
#include <dve/AVCtl/AVCtl.h>
#include <dve/AVCtl/video.h>
#include <dve/AVCtl/audio.h>
#include <dve/AVCtl/transport.h>
#include <dve/AVCtl/tones.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "menu.h"

enum {True=1, False=0};

static int setTransportPort(DveAVSession session, DveAVTransportTag tag);

#define MAXMEMBERS 5
struct {
  int numMembers;          /* Number of other members for sending/receiving */
  
  int index[MAXMEMBERS];   /* Member indices of each of these members */
  int address[MAXMEMBERS]; /* IP Addresses of each of these members */
  char *name[MAXMEMBERS];  /* Host Name of each of thse members */
  DveAVDomainMask masks[MAXMEMBERS]; /* Do we want to receive audio/video? */
  DveAVDomainMask mask;    /* Have we disabled audio or video or both? */

  int sending;
  int currentMember;       /* "Current" member for modify member routines */
} state;

static char s[80];

static DveAVDomain getDomain(DveAVDomainMask mask);
static DveAVMemberInfo *getMemberInfo(char *hostName, int *address);
static DveAVTransportArg getArg(int index, u_int16 port);

int addMember(DveAVSession session)
{
  DveAVMemberInfo *info;
  int address;
  int i;


  if (state.numMembers >= MAXMEMBERS) {
    fprintf(stderr, "Currrently we support only %d members\n", MAXMEMBERS);
    return True;
  }

  fprintf(stderr, "Enter the host name of the member to add-> ");
  gets(s);

  if (!(info = getMemberInfo(s, &address))) {
    fprintf(stderr, "That was an invalid hostname.\n");
    return True;
  }
  
  /* Is this member already in our list? */
  for (i = 0; i < state.numMembers; i++)
    if (state.address[i] == address) {
      fprintf(stderr, "Hostname %s already added.\n", s);
      return True;
    }

  state.index[state.numMembers] = dveAVAddMember(session, info);
  state.address[state.numMembers] = address;
  state.name[state.numMembers] = (char *) strdup(s);


  /* Enable audio and video functions for this member */
  dveAVEnableMember(session, state.index[state.numMembers],
		    dveAVDomainAudioVideo,
		    dveAVSendModeAll);
  state.numMembers++;

  fprintf(stderr, 
	  "\nSuccessfully added member. Member index for this member is %d\n",
	  state.index[state.numMembers-1]);

  return True;
}


int removeMember(DveAVSession session)
{
  int i, k;
  int icopy[MAXMEMBERS], acopy[MAXMEMBERS];
  char *ncopy[MAXMEMBERS];
  struct hostent *h;
  int index;
  struct in_addr in;


  fprintf(stderr, "Currently chosen members are :\n");
  for (i = 0; i < state.numMembers; i++) {
    in.s_addr = state.address[i];
    fprintf(stderr, "%d. %s (%s)\n", i+1, state.name[i], 
	    inet_ntoa(in));
  }

  fprintf(stderr, "Enter the index of the member you wish to remove-> ");
  gets(s);

  index = atoi(s)-1;
  
  if (index < 0 || index >= state.numMembers) {
    fprintf(stderr, "That was an invalid index.\n");
    return True;
  }

  dveAVRemoveMember(session, state.index[index]);

  for (i = 0, k = 0; i < state.numMembers; i++) 
    if (i != index) {
      icopy[k] = state.index[i];
      acopy[k] = state.address[i];
      ncopy[k++] = state.name[i];
    }
  
  state.numMembers--;
  for (i = 0; i < state.numMembers; i++) {
    state.index[i] = icopy[i];
    state.address[i] = acopy[i];
    state.name[i] = ncopy[i];
  }


  fprintf(stderr, "Removed member %d\n", index+1);
  return True;
}

static DveAVDomain getDomain(DveAVDomainMask mask)
{
  if ((mask & dveAVDomainMaskAudio) && (mask & dveAVDomainMaskVideo))
    return dveAVDomainAudioVideo;
  else if (mask & dveAVDomainMaskAudio)
    return dveAVDomainAudio;
  else if (mask & dveAVDomainMaskVideo)
    return dveAVDomainVideo;
  else
    return dveAVDomainInvalid;

}
  
int start(DveAVSession session)
{
  if (state.sending) {
    fprintf(stderr, "Already sending.\n");
    return True;
  }

  if (getDomain(state.mask) == dveAVDomainInvalid) {
    fprintf(stderr, "You seem to have both audio and video disabled.\n");
    return True;
  }

  dveAVStart(session, getDomain(state.mask));
  state.sending = True;
  return True;
}

int quit(DveAVSession session)
{
  dveAVCloseSession(session);
  return False;
}

int stop(DveAVSession session)
{
  dveAVStop(session, getDomain(state.mask));
  state.sending = False;
  fprintf(stderr, "Stopped.\n");
  return True;
}

int pause(DveAVSession session)
{
  dveAVPause(session, getDomain(state.mask), True);
  fprintf(stderr, "Paused.\n");
  return True;
}

int resume(DveAVSession session)
{
  dveAVPause(session, getDomain(state.mask), False);
  fprintf(stderr, "Resumed.\n");
  return True;
}

int startVideoMonitor(DveAVSession session)
{
  dveAVStartMonitor(session, dveAVDomainVideo);
  fprintf(stderr, "Monitor Started.\n");
  return False;
}

int stopVideoMonitor(DveAVSession session)
{
  dveAVStopMonitor(session, dveAVDomainVideo);
  fprintf(stderr, "Monitor stopped.\n");
  return False;
}


int disableVideo(DveAVSession session)
{
  if (state.sending)
    dveAVStop(session, dveAVDomainVideo);
  state.mask &= ~dveAVDomainMaskVideo;

  fprintf(stderr, "Disabled video.\n");
  return True;
}

int enableVideo(DveAVSession session)
{
  if (state.sending)
    dveAVStart(session, dveAVDomainVideo);

  state.mask |= dveAVDomainMaskVideo;
  fprintf(stderr, "Enabled video.\n");
  return True;
}

int disableAudio(DveAVSession session)
{
  if (state.sending)
    dveAVStop(session, dveAVDomainAudio);
  state.mask &= ~dveAVDomainMaskAudio;

  fprintf(stderr, "Disabled Audio.\n");
  return True;
}

int enableAudio(DveAVSession session)
{
  if (state.sending)
    dveAVStart(session, dveAVDomainAudio);

  state.mask |= dveAVDomainMaskAudio;

  fprintf(stderr, "Enabled audio.\n");
  return True;
}

void init()
{
  state.numMembers = 0;
  state.mask = (dveAVDomainMaskVideo | dveAVDomainMaskAudio);
  state.sending = False;
}

int initial(DveAVSession session)
{
  return True;
}

int show(DveAVSession session)
{
  dveAVShowBoard(session, dveAVDomainVideo);

}

int hide(DveAVSession session)
{
  dveAVHideBoard(session, dveAVDomainVideo);
}

int modifyInitial(DveAVSession session)
{
  int i;
  int index;
  extern char modifyMemberString[];
  struct in_addr in;

  strcpy(modifyMemberString, "Modify a member");
  fprintf(stderr, "Currently chosen members are :\n");
  for (i = 0; i < state.numMembers; i++) {
    in.s_addr = state.address[i];
    fprintf(stderr, "%d. %s (%s)\n", i+1, state.name[i], 
	    inet_ntoa(in));
  }
  
  fprintf(stderr, 
	  "Enter the index of the member whose state you wish to alter -> ");
  gets(s);

  index = atoi(s)-1;
  
  if (index < 0 || index >= state.numMembers) {
    fprintf(stderr, "That was an invalid index.\n");
    state.currentMember = -1;
    return False;
  }

  state.currentMember = index;
  sprintf(modifyMemberString, "Modify info for host %s", state.name[index]);
  return True;
}

int enableMember(DveAVSession session)
{
  DveAVDomain domain;
  DveAVSendMode mode;
  int index = state.currentMember;
  int i;

  if (index < 0) {
    fprintf(stderr, "You have not chosen a member correctly.\n");
    return False;
  }

  fprintf(stderr, "Currently %s for audio and %s for video\n",
	  (state.masks[index] & dveAVDomainMaskAudio) ? "Enabled" : "Disabled",
	  (state.masks[index] & dveAVDomainMaskVideo) ? "Enabled" : "Disabled"
	  );

  fprintf(stderr, 
	  "Enter 1 to enable audio,\n"
	  "\t2 to enable video,\n"
	  "\t3 to enable both audio and video-> ");

	  
  gets(s);

  i = atoi(s);
  if (i < 1 || i > 3) {
    fprintf(stderr, "That was an invalid choice.\n");
    return;
  }

  domain = 0;
  
  if (i == 1)
    domain = dveAVDomainAudio;
  else if (i == 2)
    domain = dveAVDomainVideo;
  else
    domain = dveAVDomainAudioVideo;

  fprintf(stderr, 
	  "Enter 1 if you wish to enable sending functions,\n"
	  "2 if you wish to enable receiving functions,\n"
	  "3 for both sending and receiving functions-> ");
  gets(s);

  i = atoi(s);
  
  if (i < 1 || i > 3) {
    fprintf(stderr, "That was an invalid choice.\n");
    return;
  }

  if (i == 1)
    mode = dveAVSendModeSend;
  else if (i == 2)
    mode = dveAVSendModeReceive;
  else if (i == 3)
    mode = dveAVSendModeAll;

  dveAVEnableMember(session, index, domain, mode);
  return False;
}

int disableMember(DveAVSession session)
{
  DveAVDomain domain;
  DveAVSendMode mode;
  int index = state.currentMember;
  int i;

  if (index < 0) {
    fprintf(stderr, "You have not chosen a member correctly.\n");
    return False;
  }

  fprintf(stderr, "Currently %s for audio and %s for video\n",
	  (state.masks[index] & dveAVDomainMaskAudio) ? "Enabled" : "Disabled",
	  (state.masks[index] & dveAVDomainMaskVideo) ? "Enabled" : "Disabled"
	  );

  fprintf(stderr, 
	  "Enter 1 to disable audio,\n"
	  "\t2 to disable video,\n"
	  "\t3 to disable both audio and video-> ");

	  
  gets(s);

  i = atoi(s);
  if (i < 1 || i > 3) {
    fprintf(stderr, "That was an invalid choice.\n");
    return;
  }

  domain = 0;
  
  if (i == 1)
    domain = dveAVDomainAudio;
  else if (i == 2)
    domain = dveAVDomainVideo;
  else
    domain = dveAVDomainAudioVideo;

  fprintf(stderr, 
	  "Enter 1 if you wish to enable sending functions,\n"
	  "2 if you wish to enable receiving functions,\n"
	  "3 for both sending and receiving functions-> ");
  gets(s);

  i = atoi(s);
  
  if (i < 1 || i > 3) {
    fprintf(stderr, "That was an invalid choice.\n");
    return;
  }

  if (i == 1)
    mode = dveAVSendModeSend;
  else if (i == 2)
    mode = dveAVSendModeReceive;
  else if (i == 3)
    mode = dveAVSendModeAll;

  dveAVDisableMember(session, index, domain, mode);
  return False;
}

int setSendingTransport(DveAVSession session)
{
  int port;
  int type;
  DveAVTransport transport;
  DveAVTransportArg arg;
  DveAVDomain domain;
  int address;

  fprintf(stderr, "Enter 1 for video, 2 for audio-> ");
  gets(s);
  type = atoi(s);

  if (type < 1 || type > 2) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }

  domain = (type == 1) ? dveAVDomainVideo : dveAVDomainAudio;

  fprintf(stderr, "Enter 1 for unicast UDP,\n"
	  "\t2 for multicast UDP-> ");

  gets(s);
  type = atoi(s);

  if (type < 0 || type > 2) {
    fprintf(stderr, "That was an invalid type.\n");
    return False;
  }

  if (type == 2) {
    fprintf(stderr, "Enter a multicast group address in dot format-> ");
    gets(s);
    address = inet_addr(s);
  }

  fprintf(stderr, 
	  "Enter an IP port number to send to (in host byte order)-> ");
  gets(s);

  port = atoi(s);

  if (port <= 0) {
    fprintf(stderr, 
	    "We currently do not support arbitrary ports on the\n"
	    "\tSending side.\n");
    return False;
  }
  
  port = htons(port);
  arg = getArg(state.currentMember, port);

  if (type == 2) {
    arg.transportType = dveAVTransportTypeMulticastUDP;
    ((DveAVTransportArgIP *)arg.arg)->address.v3.s_addr = address;
  }

  transport.remote = arg;
  dveAVSetTransport(session, state.currentMember, domain,
		    dveAVSendModeSend,
		    &transport);
  return False;
}

int setReceivingTransport(DveAVSession session)
{
  u_int16 port;
  int type;
  DveAVTransport transport;
  DveAVTransportArg arg;
  DveAVDomain domain;
  int address;

  fprintf(stderr, "Enter 1 for video, 2 for audio-> ");
  gets(s);
  type = atoi(s);

  if (type < 1 || type > 2) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }

  domain = (type == 1) ? dveAVDomainVideo : dveAVDomainAudio;

  fprintf(stderr, "Enter 1 for unicast UDP,\n"
	  "\t2 for multicast UDP-> ");

  gets(s);
  type = atoi(s);

  if (type < 0 || type > 2) {
    fprintf(stderr, "That was an invalid type.\n");
    return False;
  }

  if (type == 2) {
    fprintf(stderr, "Enter a multicast group address in dot format-> ");
    gets(s);
    address = inet_addr(s);
  }

  fprintf(stderr, "Enter an IP port number to send to. Enter zero if you\n"
	  "wish to bind to an arbitrary port-> ");
  gets(s);

  port = atoi(s);

  port = htons(port);
  arg = getArg(state.currentMember, port);
  if (type == 2) {
    arg.transportType = dveAVTransportTypeMulticastUDP;
    ((DveAVTransportArgIP *) arg.arg)->address.v3.s_addr = address;
  }

  transport.remote = arg;  
  if (dveAVSetTransport(session, state.currentMember, domain,
			 dveAVSendModeReceive, &transport) != dveAVOK) {
    fprintf(stderr, "Cannot set transport for member %d", 
	    state.currentMember);
    return False;
  }

  {
    u_int16 port = ntohs(((DveAVTransportArgIP *)transport.remote.arg)->port);
    
    fprintf(stderr, 
	    "Opened transport on port %d, %d in network byte order.\n", 
	    port, ((DveAVTransportArgIP *)transport.remote.arg)->port);
  }

  fprintf(stderr, "Press Enter to continue-> ");
  gets(s);

  return False;
}

void final(DveAVSession session)
{
  char s[80];
  fprintf(stderr, "Press enter to continue-> ");
  gets(s);
}

static DveAVMemberInfo *getMemberInfo(char *hostName, int *addr)
{
  struct hostent *h;
  static DveAVMemberInfo info;
  static DveAVNetworkAddress address;

  if ((h = gethostbyname(hostName)) == 0)
    return 0;

  address.interfaceType = dveAVInterfaceTypeIP;
  *addr = ((DveAVNetworkAddressIP *) address.address)->v3.s_addr = 
    ((struct in_addr *) h->h_addr)->s_addr;

  strcpy(info.name, hostName);
  return &info;
}


static DveAVTransportArg getArg(int index, u_int16 port)
{
  static DveAVTransportArg arg;
  DveAVTransportArgIP *ip;
  
  arg.interfaceType = dveAVInterfaceTypeIP;
  arg.transportType = dveAVTransportTypeUnicastUDP;
  arg.size = sizeof(DveAVTransportArgIP);

  ip = (DveAVTransportArgIP *) arg.arg;
  ip->address.v3.s_addr = state.address[index];
  ip->port = port;

  return arg;
}

int startViewer(DveAVSession session)
{
  int port;
  int i;
  int index;
  DveAVTransport transport;

  fprintf(stderr, "Currently chosen members are :\n");
  for (i = 0; i < state.numMembers; i++) 
    fprintf(stderr, "%d. %s (%s)\n", i+1, state.name[i], 
	    inet_ntoa(state.address[i]));

  fprintf(stderr, 
    "Enter the index of the member from whom you wish to receive video -> ");

  gets(s);

  index = atoi(s)-1;
  
  if (index < 0 || index >= state.numMembers) {
    fprintf(stderr, "That was an invalid index.\n");
    return True;
  }

  transport.remote = getArg(index, 0);
  dveAVSetTransport(session, state.index[index], 
		    dveAVDomainVideo, dveAVSendModeSend,
		    &transport);
  return True;
}


/* The following are video menu callbacks */
int changeVideoBoard(DveAVSession session)
{
  int i;
  DveAVVideoArgBoardInfo info;
  int id;

  clear();
  if (dveAVGetParameter(session, dveAVDomainVideo, dveAVVideoTagBoardInfo,
			sizeof(info), &info) != dveAVOK) {
    fprintf(stderr, 
	    "An internal error occurred. Unable to continue this option.\n");

    return True;
  }

  fprintf(stderr, "The following boards are available:\n");

  for (i = 0; i < info.numBoards; i++) 
    fprintf(stderr, "* %s : Board ID %d\n", info.boards[i].name,
	    info.boards[i].boardId);

  fprintf(stderr, "\nEnter the id of the board you are interested in-> ");
  gets(s);

  id = atoi(s);

  if (dveAVSetIntegerParameter(session, dveAVDomainVideo, 
			       dveAVVideoTagCurrentBoard,
			       id) != dveAVOK) {
    fprintf(stderr, 
	    "Cannot change board to requested board id.\n");
    
    return True;
  }
			
  return True;
}


int changeVideoCompression(DveAVSession session)
{
  u_int32 id;
  DveAVVideoArgCompressionInfo cinfo;
  DveAVVideoArgCompression cmp;
  int i;

  /* You can cache the current board id, I was just too lazy to do it... */
  clear();
  if (dveAVGetIntegerParameter(session, dveAVDomainVideo, 
			       dveAVVideoTagCurrentBoard,
			       &id) != dveAVOK) {
    fprintf(stderr, 
	    "An internal error occurred. Unable to continue this option.\n");
    
    return True;
  }
  
  cinfo.boardId = id;
  if (dveAVGetParameter(session, dveAVDomainVideo, 
			dveAVVideoTagCompressionInfo,
			sizeof(cinfo), &cinfo) != dveAVOK) {
    fprintf(stderr, 
	    "An internal error occurred. Unable to continue this option.\n");
    
    return True;
  }
   
  fprintf(stderr, "Available compressions are :\n");
  for (i = 0; i < cinfo.numCompressions; i++)
    fprintf(stderr, "* %s\n", cinfo.compressions[i].codec);

  fprintf(stderr, 
	  "Enter the four-character code of the compression of your choice-> "
	  );
  gets(s);

  strncpy(cmp.codec, s, 4);
  cmp.boardId = id;
  cmp.codec[4] = 0;

  if (dveAVSetParameter(session, dveAVDomainVideo, 
			dveAVVideoTagCurrentCompression,
			sizeof(cmp), &cmp) != dveAVOK) {
    fprintf(stderr, 
	    "%s was an invalid compression.\n", cmp.codec);
    
    return True;
  }	

  return True;
}


int exitVideoMenu(DveAVSession session)
{
  return False;
}

static struct {
  DveAVVideoTag tag;
  char *name;
} params[] = {
  {dveAVVideoTagFrameRate, "Frame Rate"},
  {dveAVVideoTagBandwidth, "Bandwidth",},
  {dveAVVideoTagQuantization, "Quantization"},
  {dveAVVideoTagBrightness, "Brightness"},
  {dveAVVideoTagContrast, "Contrast"},
  {dveAVVideoTagHue, "Hue"},
  {dveAVVideoTagSaturation, "Saturation"},
  {dveAVVideoTagSharpness, "Sharpness"},
  {dveAVVideoTagFrameSize, "FrameSize"},
  {dveAVVideoTagFrameHeight, "FrameHeight"},
  {dveAVVideoTagFrameWidth, "FrameWidth"},
  {dveAVVideoTagQuality, "Quality"},
  {dveAVVideoTagInput, "Input"},
  {dveAVVideoTagFormat, "Format"},
  {dveAVVideoTagDoGamma, "Do Gamma Correction"},
  {dveAVVideoTagDoCalibration, "Do Calibration"},
  {dveAVVideoTagDoRefresh, "Do Refresh"},
  {dveAVVideoTagFavor, "Favor"},
 };

const numParams = sizeof(params) / sizeof(params[0]);

int boardParamsInitial(DveAVSession session)
{
  int i;
  int half = numParams/2;

  fprintf(stderr, "Available parameters are :\n");
  for (i = 0; i < half; i++)
    fprintf(stderr, "%d. %s\t%d. %s\n", i+1, params[i].name,
	    i+half+1, params[i+half].name);
    
  fprintf(stderr, "\n");
  return True;
}

int getParam(DveAVSession session)
{
  int choice;
  u_int32 value;
  DveAVError ret;

  fprintf(stderr, "\nEnter the code of the param of your choice-> ");
  gets(s);

  choice = atoi(s);
  
  choice--;
  if (choice < 0 || choice >= numParams) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }
  
  if ((ret = dveAVGetIntegerParameter(session, dveAVDomainVideo, 
				      params[choice].tag,
				      &value)) != dveAVOK) {
    fprintf(stderr, "Cannot get value for parameter %d : %d\n", 
	    params[choice].tag, ret);
    return True;
  }

  fprintf(stderr, "Value of this parameter is %d\n", value);
  return True;
}

int setParam(DveAVSession session)
{
  int choice;
  int value;

  fprintf(stderr, "Enter the code of the param of your choice-> ");
  gets(s);

  choice = atoi(s);
  
  choice--;
  if (choice < 0 || choice >= numParams) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }
  
  fprintf(stderr, "Enter new value for this param-> ");
  gets(s);
  
  value = atoi(s);
  if (dveAVSetIntegerParameter(session, dveAVDomainVideo, params[choice].tag,
			       value) != dveAVOK) {
    fprintf(stderr, "Cannot set value for this parameter!\n");
    return True;
  }

  return True;
}

  
int exitParamsMenu(DveAVSession session)
{
  return False;
}


/* Audio functions */

char *getInputPortStr(DveLocalAudioInputDeviceTag tag)
{
  switch (tag) {
  case dveLocalAudioInputDeviceTagMicrophone:
    return "Microphone";

  case dveLocalAudioInputDeviceTagLineIn:
    return "Line In";

  case dveLocalAudioInputDeviceTagAltMicrophone:
    return "Alt Microphone";

  case dveLocalAudioInputDeviceTagAltLineIn:
    return "Alt Line In";

  default:
    return "Unknown";
  }
}


char *getOutputPortStr(int mask)
{
  static char ret[256];
  int len;

  ret[0] = 0;

  if (mask & dveLocalAudioOutputDeviceMaskSpeaker)
    strcat(ret, " Speaker,");
  
  if (mask & dveLocalAudioOutputDeviceMaskLineOut)
    strcat(ret, " Line Out,");
  
  if (mask & dveLocalAudioOutputDeviceMaskHeadphones)
    strcat(ret, " Head Phones,");

  if (mask & dveLocalAudioOutputDeviceMaskAltSpeaker)
    strcat(ret, " Alt Speaker,");

  if (mask & dveLocalAudioOutputDeviceMaskAltLineOut)
    strcat(ret, " Alt Line Out,");
  
  if (mask & dveLocalAudioOutputDeviceMaskAltHeadphones)
    strcat(ret, " Alt Head Phones,");

  /* Remove the trailing comma */
  if ((len = strlen(ret)) > 0) 
    ret[len-1] = 0;

  return ret;
}

int setInputPort(DveAVSession session) 
{
  unsigned int input;
  int choice;

  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagInputPort, &input) != dveAVOK) 
    fprintf(stderr, "Cannot get current Input Port value.\n\n");
  else 
    fprintf(stderr, "Current Input port is : %s\n\n", 
	    getInputPortStr(input));

  fprintf(stderr, "Enter 1 for Microphone,\n"
	  "\t2 for Line In-> ");

  gets(s);
  
  choice = atoi(s);

  if (choice < 1 || choice > 2) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }

  switch(choice) {
  case 1:
    input = dveLocalAudioInputDeviceTagMicrophone;
    break;
    
  case 2:
    input = dveLocalAudioInputDeviceTagLineIn;
    break;
  }

  if (dveAVSetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagInputPort, input) != dveAVOK) 
    fprintf(stderr, "Cannot set value for input port!\n");

  return False;
}  
    
int setOutputPort(DveAVSession session) 
{
  unsigned int outputs;
  int choice, add;

  if (dveAVGetIntegerParameter(session, dveAVDomainAudio,
			       dveAVAudioTagOutputPort, &outputs) != dveAVOK) {
    fprintf(stderr, "Cannot get current outputs. Sorry, Cannot continue.\n");
    return False;
  }

  fprintf(stderr, "Current outputs are : %s\n\n", getOutputPortStr(outputs));

  fprintf(stderr, "Enter 1 for speaker,\n"
	  "2 for line out,\n"
	  "3 for head phones-> ");

  gets(s);

  choice = atoi(s);

  if (choice < 1 || choice > 3) {
    fprintf(stderr, "That Was an Invalid choice.\n");
    return False;
  }

  fprintf(stderr, "Enter 1 to add your choice as an output device; \n"
	  "Enter 2 to remove your choice as an output device-> ");
  
  gets(s);

  add = atoi(s);

  if (add < 1 || add > 2) {
    fprintf(stderr, "That was an invalid choice.\n");
    return False;
  }

  add = (add == 1);

  switch(choice) {
  case 1:
    if (add) 
      outputs |= dveLocalAudioOutputDeviceMaskSpeaker;
    else
      outputs &= ~dveLocalAudioOutputDeviceMaskSpeaker;
    
    break;

  case 2:
    if (add)
      outputs |= dveLocalAudioOutputDeviceMaskLineOut;
    else
      outputs &= ~dveLocalAudioOutputDeviceMaskLineOut;
    
    break;

  case 3:
    if (add)
      outputs |= dveLocalAudioOutputDeviceMaskHeadphones;
    else
      outputs &= ~dveLocalAudioOutputDeviceMaskHeadphones;
    break;
  }

  dveAVSetIntegerParameter(session, dveAVDomainAudio, 
			   dveAVAudioTagOutputPort, outputs);
  
  return False;
}

    
int setPlayVol(DveAVSession session)
{
  unsigned int vol;
  
  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagPlayVolume, &vol) != dveAVOK) 
    fprintf(stderr, "Cannot get current play volume.\n\n");
  else
    fprintf(stderr, "Current play volume is %d\n\n", vol);

  fprintf(stderr, "Enter new volume-> ");

  gets(s);

  vol = atoi(s);

  if (vol > 0)
    dveAVSetIntegerParameter(session, dveAVDomainAudio,
			     dveAVAudioTagPlayVolume, vol);

  return False;
}


int setSendVol(DveAVSession session)
{
  unsigned int vol;

  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagMicVolume, &vol) != dveAVOK) 
    fprintf(stderr, "Cannot get current send volume.\n\n");
  else
    fprintf(stderr, "Current send volume is %d\n\n", vol);

  fprintf(stderr, "Enter new volume-> ");

  gets(s);

  if ((vol = atoi(s)) > 0)
    dveAVSetIntegerParameter(session, dveAVDomainAudio,
			     dveAVAudioTagMicVolume, vol);

  return False;
}

int setSilence(DveAVSession session)
{
  unsigned int silence;

  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagSilenceSensor, &silence) != dveAVOK) 
    fprintf(stderr, "Cannot get current silence sensor level.\n\n");
  else
    fprintf(stderr, "Current silence sensor level is %d\n\n", silence);

  fprintf(stderr, "Enter new level-> ");

  gets(s);

  silence = atoi(s);

  dveAVSetIntegerParameter(session, dveAVDomainAudio,
			   dveAVAudioTagSilenceSensor, silence);

  return False;
}

  
int setEcho(DveAVSession session)
{
  unsigned int echo;

  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagEchoCancellation, &echo) != 
      dveAVOK) 
    fprintf(stderr, "Cannot get current echo cancellation value.\n\n");
  else
    fprintf(stderr, "Current echo cancellation value is %d\n\n", echo);

  fprintf(stderr, "Enter new value-> ");

  gets(s);

  echo = atoi(s);

  dveAVSetIntegerParameter(session, dveAVDomainAudio,
			   dveAVAudioTagEchoCancellation, echo);

  return False;
}

int setsync(DveAVSession session)
{
  unsigned int sync;
  
  if (dveAVGetIntegerParameter(session, dveAVDomainAudio, 
			       dveAVAudioTagAVSync, &sync) != dveAVOK) 
    fprintf(stderr, "Cannot get current av sync value.\n\n");
  else
    fprintf(stderr, "Current av sync value is %d\n\n", sync);

  fprintf(stderr, "Enter new value-> ");

  gets(s);

  sync = atoi(s);

  dveAVSetIntegerParameter(session, dveAVDomainAudio,
			   dveAVAudioTagAVSync, sync);

  return False;
}

int exitAudioMenu(DveAVSession session)
{
  return False;
}

int listInterfaces(DveAVSession session)
{
  DveAVTransportArgAvailInterfaces arg;
  int i;

  if (dveAVGetParameter(session, dveAVDomainTransport, 
			dveAVTransportTagAvailInterfaces,
			sizeof(arg), &arg) != dveAVOK) {
    fprintf(stderr, "Cannot get transport info.\n");
    return False;
  }

  
  fprintf(stderr, "\nThere are %d network interfaces.\n\n", arg.numIfaces);
  for (i = 0; i < arg.numIfaces; i++) {

    DveAVNetworkIfaceIP *ip = (DveAVNetworkIfaceIP *)
      arg.ifaces[i].iface.iface; /* Whew! */

    fprintf(stderr, "******************** Interface %d ********************\n",
	    i+1);

    fprintf(stderr, "Host address : %s\n", inet_ntoa(ip->addr.v3.s_addr));
    fprintf(stderr, 
	    "Broadcast address : %s\n", inet_ntoa(ip->broadcast.v3.s_addr));
    
    fprintf(stderr, "Host Name : %s\n", ip->hostName);
    fprintf(stderr, "Interface Name : %s\n", ip->ifaceName);
  }
 
  fprintf(stderr, "Press Enter to continue-> ");
  gets(s);

  return False;
}

int showAudioInterface(DveAVSession session)
{
  dveAVShowBoard(session, dveAVDomainAudio);
  return True;
}

int hideAudioInterface(DveAVSession session)
{
  dveAVHideBoard(session, dveAVDomainAudio);
  return True;
}

static int getVideoBoardId(DveAVSession session)
{
  int i;
  DveAVVideoArgBoardInfo info;
  int id;
  
  clear();
  if (dveAVGetParameter(session, dveAVDomainVideo, dveAVVideoTagBoardInfo,
			sizeof(info), &info) != dveAVOK) {
    fprintf(stderr, 
	    "An internal error occurred. Unable to continue this option.\n");
    
    return -1;
  }
  
  fprintf(stderr, "The following boards are available:\n");
  
  for (i = 0; i < info.numBoards; i++) 
    fprintf(stderr, "* %s : Board ID %d\n", info.boards[i].name,
	    info.boards[i].boardId);
  
  fprintf(stderr, "\nEnter the id of the board you are interested in-> ");
  gets(s);
  
  id = atoi(s);
  return id;

}

static int getAudioBoardId(DveAVSession session)
{
  return 0;
}

int loadVideoPrefs(DveAVSession session)
{
  int id = getVideoBoardId(session);

  dveAVLoadPrefs(session, dveAVDomainVideo, id);
  return True;
}

int saveVideoPrefs(DveAVSession session)
{
  int id = getVideoBoardId(session);

  dveAVSavePrefs(session, dveAVDomainVideo, id);
  return True;
}

int loadVideoDefaults(DveAVSession session)
{
  int id = getVideoBoardId(session);

  dveAVLoadDefaults(session, dveAVDomainVideo, id);
  return True;
}

int loadAudioPrefs(DveAVSession session)
{
  int id = getAudioBoardId(session);

  dveAVLoadPrefs(session, dveAVDomainAudio, id);
  return True;
}

int saveAudioPrefs(DveAVSession session)
{
  int id = getAudioBoardId(session);

  dveAVSavePrefs(session, dveAVDomainAudio, id);
  return True;
}

int loadAudioDefaults(DveAVSession session)
{
  int id = getAudioBoardId(session);

  dveAVLoadDefaults(session, dveAVDomainAudio, id);
  return True;
}

static int fileId = -1, dtmfId = -1;

int playAudioFile(DveAVSession session)
{
  int i;
  int choice;
  char fileName[256];
  int index;
  int loop, sleepTime;
  DveAVTransmitMode tmode;
  DveAVMixMode mmode;
  int gain;

 start:
  clear();
  
  fprintf(stderr, "Enter the full pathname of the file you wish to play-> ");
  gets(fileName);


  fprintf(stderr, "Enter 1 to loop, 0 to play tone once-> ");
  gets(s);
  
  loop = atoi(s);

  if (loop) {
    fprintf(stderr, "Enter time to sleep between successive playbacks-> ");
    gets(s);
    sleepTime = atoi(s);
  }

 again:
  fprintf(stderr, "Enter 1 to playback tone with incoming audio,\n"
	  "\t2 to transmit tone to network,\n"
	  "\t3 for both-> ");
  gets(s);
  
  choice = atoi(s);

  if (choice <= 0 || choice > 3) {
    fprintf(stderr, "That was an invalid choice. Try again!\n\n");
    goto again;
  }
  
  switch (choice) {
  case 1:
    tmode = dveAVTransmitModeLocal;
    break;

  case 2:
    tmode = dveAVTransmitModeNetwork;
    break;

  case 3:
    tmode = dveAVTransmitModeAll;
    break;
  }

 againMix:
  fprintf(stderr, "Enter 1 to mix tone with incoming audio\n"
	  "2 to mix with outgoing audio,\n"
	  "3 to mix with both incoming and outgoing audio\n"
	  "4 to not mix-> ");
  gets(s);

  choice = atoi(s);

  if (choice <= 0 || choice > 4) {
    fprintf(stderr, "That was an invalid choice. Try again!\n\n");
    goto againMix;
  }
  
  switch (choice) {
  case 1:
    mmode = dveAVMixModeLocal;
    break;
    
  case 2:
    mmode = dveAVMixModeNetwork;
    break;

  case 3:
    mmode = dveAVMixModeAll;
    break;

  case 4:
    mmode = dveAVMixModeNone;
    break;
  }
  
  fprintf(stderr, 
	  "Enter gain (from 0 to 100) to add to the file playback.\n"
	  "A number of 50 is no gain. We add 0.5 db of gain for every\n"
	  "number above 50, and -0.5 db for every number below 50 -->");
  gets(s);
  gain = atoi(s);
  
  if (gain < 0 || gain > 100) {
    gain = 50;
    fprintf(stderr, "That was an invalid gain. Gain set to 50\n");
    fprintf(stderr, "Press Enter to continue-> ");
    gets(s);
  }

  dveAVSetFileGain(session, dveAVGainTypeFToS, gain);

  if ((fileId = dveAVPlayFile(session, dveAVDomainAudio, fileName, 0,
			      sleepTime,
			      tmode,
			      (loop) ? 
			      dveAVPlayModeRepeated : dveAVPlayModeOnce,
			      mmode)) < 0) {
    fprintf(stderr, 
       "Cannot play file. Check whether the path you entered is correct.\n");
    fprintf(stderr, "Press enter to continue-> ");
    gets(s);
  }

  return False;
}

int genDTMFTone(DveAVSession session)
{
  char string[48];
  int idt, duration;
  int choice;
  DveAVTransmitMode tmode;
  int loop;
  DveAVMixMode mmode;
  int gain;

  fprintf(stderr, "Enter the DTMF string you wish to play-> ");
  gets(s);
  strcpy(string, s);

  fprintf(stderr, "Enter the duration of each tone-> ");
  gets(s);
  duration = atoi(s);

  fprintf(stderr, "Enter the inter-digit time-> ");
  gets(s);
  idt = atoi(s);

  fprintf(stderr, "Enter 1 to loop, 0 to play tone once-> ");
  gets(s);
  
  loop = atoi(s);

 again:
  fprintf(stderr, "Enter 1 to playback tone with incoming audio,\n"
	  "\t2 to transmit tone to network,\n"
	  "\t3 for both-> ");
  gets(s);
  
  choice = atoi(s);

  if (choice <= 0 || choice > 3) {
    fprintf(stderr, "That was an invalid choice. Try again!\n\n");
    goto again;
  }
  
  switch (choice) {
  case 1:
    tmode = dveAVTransmitModeLocal;
    break;

  case 2:
    tmode = dveAVTransmitModeNetwork;
    break;

  case 3:
    tmode = dveAVTransmitModeAll;
    break;
  }

 againMix:
  fprintf(stderr, "Enter 1 to mix tone with incoming audio\n"
	  "2 to mix with outgoing audio,\n"
	  "3 to mix with both incoming and outgoing audio\n"
	  "4 to not mix-> ");
  gets(s);

  choice = atoi(s);

  if (choice <= 0 || choice > 4) {
    fprintf(stderr, "That was an invalid choice. Try again!\n\n");
    goto againMix;
  }
  
  switch (choice) {
  case 1:
    mmode = dveAVMixModeLocal;
    break;
    
  case 2:
    mmode = dveAVMixModeNetwork;
    break;

  case 3:
    mmode = dveAVMixModeAll;
    break;

  case 4:
    mmode = dveAVMixModeNone;
    break;
  }

  fprintf(stderr, 
	  "Enter gain (from 0 to 100) to add to the file playback.\n"
	  "A number of 50 is no gain. We add 0.5 db of gain for every\n"
	  "number above 50, and -0.5 db for every number below 50 -->");
  gets(s);
  gain = atoi(s);
  
  if (gain < 0 || gain > 100) {
    gain = 50;
    fprintf(stderr, "That was an invalid gain. Gain set to 50\n");
    fprintf(stderr, "Press Enter to continue-> ");
    gets(s);
  }

  dveAVSetFileGain(session, dveAVGainTypeDToS, gain);

  if ((dtmfId = dveAVStartDTMFTone(session, string, duration, idt, 
				   (loop) ? 
				   dveAVPlayModeRepeated : dveAVPlayModeOnce,
				   tmode, mmode)) < 0) {
    fprintf(stderr, "Cannot generate DTMF tone : Invalid dtmf string\n");
    fprintf(stderr, "Press enter to continue-> ");
    gets(s);
  }

  return False;
}

int stopAudioFile(DveAVSession session)
{
  dveAVStopFilePlayback(session, fileId);

  fprintf(stderr, "File Playback stopped. Press any key to continue-> ");
  gets(s);
  return False;
}


int stopDTMFTone(DveAVSession session)
{
  dveAVStopDTMFTone(session, dtmfId);

  fprintf(stderr, "DTMF Tone stopped. Press any key to continue-> ");
  gets(s);
  return False;
}




