#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <Xm/Xm.h>

#define OPENDVE_CONFERENCE_MANAGER_PORT     1

/* 
 * OpenDVE related includes...
 */
#include <dve/dve.h>
#include <dve/dvePluginMgr.h>

/* 
 * OpenDVE Related Variables...
 */
DveSession      dveSession;
int             pluginCount;
int             reLoop = 0;
int             initRC;

extern XtAppContext app;


/* 
 * Forward declarations...
 */
void     DealWithPluginReady(pluginObject plug);
void     DealWithPluginDeath(pluginObject plug);
void     CleanupAndExit(int sig);

main(int argc, char *argv[])
{

  initializeOpenDve();
  createInterface(argc,argv);

  XtAppMainLoop(app);

}

int initializeOpenDve()
{
  if (!(dveSession=dveOpenSession(OPENDVE_CONFERENCE_MANAGER_PORT)))
    {
      fprintf(stderr,
	      "Plugin: Could not initialize OpenDVE library.\n");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}
      
      exit(-1);
    }
  
  
  (void) signal(SIGINT, CleanupAndExit);
  
  initRC = dvePluginMgrInitialize(dveSession, NULL, 
				  DealWithPluginReady, DealWithPluginDeath);
  if (initRC < 0)
    {
      fprintf(stderr, "Error %d on dvePluginMgrInitialize().\n", 
	      dvePluginMgrErrno);
      exit(-1);
    }
  else
    {
/*
      fprintf(stderr, "dvePluginMgrInitialize() Successful.\n"); 
*/
    }
  
  pluginCount = dvePluginMgrIdentifyAvailablePlugins(dveSession);

}


int pluginStart(int pos)
{
  dvePluginMgrStartPlugin(dveSession,dvePluginList[pos],PLUGIN_MAPPED);
  return(0);
}

int pluginStartAll()
{
  dvePluginMgrStartAllPlugins(dveSession,PLUGIN_MAPPED);
  return(0);
}

int pluginStop(int pos)
{
  dvePluginMgrQuitPlugin(dveSession,dvePluginList[pos]);
  return(0);
}

int pluginStopAll()
{
  dvePluginMgrStopAllPlugins(dveSession);
  return(0);
}

int pluginHide(int pos)
{
  dvePluginMgrHidePlugin(dveSession,dvePluginList[pos]);
  return(0);
}

int pluginHideAll()
{
  dvePluginMgrHideAllPlugins(dveSession);
  return(0);
}

int pluginShow(int pos)
{
  dvePluginMgrShowPlugin(dveSession,dvePluginList[pos]);
  return(0);
}

int pluginShowAll()
{
  dvePluginMgrShowAllPlugins(dveSession);
  return(0);
}

void DealWithPluginReady(pluginObject plug)
{
  return;
}

void DealWithPluginDeath(pluginObject plug)
{
  return;
}

void CleanupAndExit(int sig)
{
  int		i;
  
  /*
   * First stop all running plugins...
   */
  dvePluginMgrStopAllPlugins(dveSession);
  
  fprintf(stderr, "\nClosing PluginMgr library and Exiting.\n");
  dvePluginMgrClose();
  dveCloseSession(dveSession);
  exit(0);
}
