/*
 *
 * Name:	ISSamplePluginRaw.c
 *
 * Description:
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <dve/dve.h>
#include <dve/confMgr.h>
#include "SamplePluginProtocol.h"

#define RAW_MAXLINE	80

extern DveSession dveSession;

samplePluginMessageStructure *rawmsg;
fd_set readfds;

void
QuitApplication(int signo)
{
   dveFree(rawmsg);
   CleanupPlugin();
   exit(0);
}

/*
 * Main()
 */
void
main(int argc, char **argv)
{
   struct sigaction act;
   struct timeval tv;
   DveConfMgrUserAddr boo;

   char buf[80];

   int gOpenDVEfd;
   int current_state;

   signal(SIGQUIT, QuitApplication);
   signal(SIGINT, QuitApplication);

   /*
    * Start up OpenDVE
    */
   if (InitializeOpenDVE(SAMPLE_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

   if (InitializePluginClipboard() < 0)
   {
      fprintf(stderr, "Initialization of OpenDVE Clipboard Failed.\n\7");
   }

   /*
    * Make room for our local raw settings
    */
   rawmsg = (samplePluginMessageStructure *)dveMallocDataPacket(dveSession, sizeof(samplePluginMessageStructure));

   if (!rawmsg)
   {
      fprintf(stderr, "Unable to allocate space...exiting.\n\7");
      exit(-1);
   }

   rawmsg->protocolVersion = VERSION;
   rawmsg->temperature = 50;
   rawmsg->musicChoices = 0;
   rawmsg->drinkSelection = 0;
	
   fprintf(stderr, "This is an example of an OpenDVE Raw Plugin.  Simply select from\n"
                   "the menu choices to see the actions distributed amongst the\n"
                   "conference members.\n");
   current_state = DisplayMainMenu();

   while(1)
   {
      gOpenDVEfd = dveSetFds(dveSession, &readfds);
      FD_SET(0, &readfds);
      tv.tv_sec = 0.5;
      if (select((gOpenDVEfd + 1), (fd_set *)&readfds, NULL, NULL, &tv) > 0)
      {
         if (FD_ISSET(0, &readfds))
         {
            if (fgets(buf, RAW_MAXLINE, stdin) != NULL)
            {
               current_state = ProcessInput(buf, current_state);
            }
            FD_CLR(0, &readfds);
         }
      }
      dveProcessIncoming(dveSession, &readfds);
   }
}
