#ifndef _INSOFT_AUDIO_UTILS_H_
#define _INSOFT_AUDIO_UTILS_H_

#ifdef __cplusplus
extern "C" {
#endif

#define MINLIN 	-32768
#define MAXLIN 	32767
#define LINCLIP(x) do { if ( x < MINLIN ) x = MINLIN ; else if ( x > MAXLIN ) x = MAXLIN; } while ( 0 )

typedef void (*AUDIO_CALLBACK)(int);

/* The shm block uses a mask so it can specify available devices, and
 *  multiple simultaneous output devices; however, the interface should
 *  assume the platform can support only one at a time, unless the capture
 *  process sets devices.output.maxActive to >1.
 */

typedef enum {
  dveLocalAudioInputDeviceTagInvalid=0,
  dveLocalAudioInputDeviceTagMicrophone,
  dveLocalAudioInputDeviceTagLineIn,
  dveLocalAudioInputDeviceTagAltMicrophone=8,
  dveLocalAudioInputDeviceTagAltLineIn
  } DveLocalAudioInputDeviceTag;

typedef enum {
  dveLocalAudioInputDeviceMaskNull=0,
  dveLocalAudioInputDeviceMaskMicrophone
    =(1<<dveLocalAudioInputDeviceTagMicrophone),
  dveLocalAudioInputDeviceMaskLineIn
    =(1<<dveLocalAudioInputDeviceTagLineIn),
  dveLocalAudioInputDeviceMaskAltMicrophone
    =(1<<dveLocalAudioInputDeviceTagAltMicrophone),
  dveLocalAudioInputDeviceMaskAltLineIn
    =(1<<dveLocalAudioInputDeviceTagAltLineIn)
  } DveLocalAudioInputDeviceMask;

typedef enum {
  dveLocalAudioOutputDeviceTagInvalid=0,
  dveLocalAudioOutputDeviceTagSpeaker,
  dveLocalAudioOutputDeviceTagHeadphones,
  dveLocalAudioOutputDeviceTagLineOut,
  dveLocalAudioOutputDeviceTagAltSpeaker=8,
  dveLocalAudioOutputDeviceTagAltHeadphones,
  dveLocalAudioOutputDeviceTagAltLineOut
  } DveLocalAudioOutputDeviceTag;

typedef enum {
  dveLocalAudioOutputDeviceMaskNull=0,
  dveLocalAudioOutputDeviceMaskSpeaker
    =(1<<dveLocalAudioOutputDeviceTagSpeaker),
  dveLocalAudioOutputDeviceMaskHeadphones
    =(1<<dveLocalAudioOutputDeviceTagHeadphones),
  dveLocalAudioOutputDeviceMaskLineOut
    =(1<<dveLocalAudioOutputDeviceTagLineOut),
  dveLocalAudioOutputDeviceMaskAltSpeaker
    =(1<<dveLocalAudioOutputDeviceTagAltSpeaker),
  dveLocalAudioOutputDeviceMaskAltHeadphones
    =(1<<dveLocalAudioOutputDeviceTagAltHeadphones),
  dveLocalAudioOutputDeviceMaskAltLineOut
    =(1<<dveLocalAudioOutputDeviceTagAltLineOut)
  } DveLocalAudioOutputDeviceMask;

typedef enum {
  dveLocalAudioCompressionTagNone=0,
  dveLocalAudioCompressionTagHalfsies /* "drop low bits" */
  } DveLocalAudioCompressionTag;

int st_ulaw_to_linear_slow( /* unsigned char ulawbyte */ );

/*
 * This is needed for querying the audio device.  
 */
typedef struct dveAudioInfo {
	struct {
	  u_int32 gain;		/* gain level: 0 - 255 */
	  u_int32 port;		/* selected I/O port (see below) */
	  u_int32 avail_ports;	/* available I/O ports (see below) */

	} play,record;

	/*
	 * Per-unit/channel information
	 */
	u_int32 monitor_gain;	/* input to output mix: 0 - 255 */
	Bool16 	output_muted;	/* non-zero if output is muted */
} dveAudioInfoType;


/*
** This macro converts from ulaw to 16 bit linear, faster.
** Input: 8 bit ulaw sample
** Output: signed 16 bit linear sample
*/
#define st_ulaw_to_linear(ulawbyte) ulaw_table[ulawbyte]

static int ulaw_table[256] = {
    -32124, -31100, -30076, -29052, -28028, -27004, -25980, -24956,
    -23932, -22908, -21884, -20860, -19836, -18812, -17788, -16764,
    -15996, -15484, -14972, -14460, -13948, -13436, -12924, -12412,
    -11900, -11388, -10876, -10364,  -9852,  -9340,  -8828,  -8316,
     -7932,  -7676,  -7420,  -7164,  -6908,  -6652,  -6396,  -6140,
     -5884,  -5628,  -5372,  -5116,  -4860,  -4604,  -4348,  -4092,
     -3900,  -3772,  -3644,  -3516,  -3388,  -3260,  -3132,  -3004,
     -2876,  -2748,  -2620,  -2492,  -2364,  -2236,  -2108,  -1980,
     -1884,  -1820,  -1756,  -1692,  -1628,  -1564,  -1500,  -1436,
     -1372,  -1308,  -1244,  -1180,  -1116,  -1052,   -988,   -924,
      -876,   -844,   -812,   -780,   -748,   -716,   -684,   -652,
      -620,   -588,   -556,   -524,   -492,   -460,   -428,   -396,
      -372,   -356,   -340,   -324,   -308,   -292,   -276,   -260,
      -244,   -228,   -212,   -196,   -180,   -164,   -148,   -132,
      -120,   -112,   -104,    -96,    -88,    -80,    -72,    -64,
       -56,    -48,    -40,    -32,    -24,    -16,     -8,      0,
     32124,  31100,  30076,  29052,  28028,  27004,  25980,  24956,
     23932,  22908,  21884,  20860,  19836,  18812,  17788,  16764,
     15996,  15484,  14972,  14460,  13948,  13436,  12924,  12412,
     11900,  11388,  10876,  10364,   9852,   9340,   8828,   8316,
      7932,   7676,   7420,   7164,   6908,   6652,   6396,   6140,
      5884,   5628,   5372,   5116,   4860,   4604,   4348,   4092,
      3900,   3772,   3644,   3516,   3388,   3260,   3132,   3004,
      2876,   2748,   2620,   2492,   2364,   2236,   2108,   1980,
      1884,   1820,   1756,   1692,   1628,   1564,   1500,   1436,
      1372,   1308,   1244,   1180,   1116,   1052,    988,    924,
       876,    844,    812,    780,    748,    716,    684,    652,
       620,    588,    556,    524,    492,    460,    428,    396,
       372,    356,    340,    324,    308,    292,    276,    260,
       244,    228,    212,    196,    180,    164,    148,    132,
       120,    112,    104,     96,     88,     80,     72,     64,
	56,     48,     40,     32,     24,     16,      8,      0 };


/* 
 * Syntax:
 *  int
 *  IS_StripQuiet(char *inBuffer, char *outBuffer, int inLength,
 *                        int sensorLevel, int supressionFactor)
 * 
 * Description: 
 * Take the inBuffer and scan through it, strip out any null sound
 * data an return the "clean" buffer in outBuffer with the return code 
 * being the length of the outBuffer.
 * 
 * Return value: 
 *      length of the output buffer
 * 
 * Implemented By:
 *   insoft_compression.c
 *
 * Author:
 *   Richard H. Pizzarro (1/19/94)
 *
 */ 
int
IS_StripQuiet(char *inBuffer, char *outBuffer, int inLength,
                        int sensorLevel, int supressionFactor);

/* 
 * Syntax:
 * int
 * IS_CompressAudioBuffer(char *InBuf, int inputLength, char *OutBuf);
 * 
 * Description: 
 * This is a compression routine that will discard the 4 LSB's and turn every
 * audio 16bit pair into an 8 (end effect: 2:1 compression with slight loss)
 * 
 * Return value: 
 *      length of the output buffer
 * 
 * Implemented By:
 *   insoft_compression.c
 *
 * Author:
 *   Richard H. Pizzarro (1/19/94)
 *
 */ 
int
IS_CompressAudioBuffer(char *InBuf, int inputLength, char *OutBuf);

/* 
 * Syntax:
 * int
 * IS_DecompressAudioBuffer(char *InBuf, int inputLength, char *OutBuf);
 * 
 * Description: 
 * This will rebuild the 16 bit pairs from the single 8bit sample. All thats
 * going on here is much bit shifting...
 *
 * Return value: 
 *      length of the output buffer
 * 
 * Implemented By:
 *   insoft_compression.c
 *
 * Author:
 *   Richard H. Pizzarro (1/19/94)
 *
 */ 
int
IS_DecompressAudioBuffer(char *InBuf, int inputLength, char *OutBuf);

/* 
 * Syntax:
 * int
 * st_linear_to_ulaw(int sample)
 * 
 * Description: 
 * This routine converts from linear to ulaw.
 *
 * Input: sample - Signed 16 bit linear sample
 * Output: 8 bit ulaw sample
 *
 * Return value: 
 *      the ulaw sample
 * 
 * Implemented By:
 *   insoft_compression.c
 *
 * Author:
 *   Richard H. Pizzarro (1/19/94)
 *
 */ 
unsigned char
st_linear_to_ulaw( int sample );


/*
 * This will open the control device and return 0 if
 * successful or a -1 on failure.
 */
int
IS_AudioInitialize(AUDIO_CALLBACK callback);


/*
 * This will close and cleanup the shutdown all audio fd's.
 */
int
IS_AudioShutdown(void);

/*
 * Open the device for reading and return a fd to the device.
 *
 *	this returns -1 on failure.
 */
int
IS_AudioOpenReadDevice(void);

/*
 * Close the read device.
 *
 *	this returns -1 on failure.
 */
int
IS_AudioCloseReadDevice(void);


/*
 * Open the device for writing and return a fd to the device.
 *
 *	this returns -1 on failure.
 */
int
IS_AudioOpenWriteDevice(void);

/*
 * Close the write device.
 *
 *	this returns -1 on failure.
 */
int
IS_AudioCloseWriteDevice(void);

/*
 * This will get the number of output devices available...
 */
int 
IS_AudioGetMaxOutputDevices(void);

/*
 * This will get a mask of the available output devices on the 
 * machine.
 */
DveLocalAudioOutputDeviceMask
IS_AudioGetAvailOutputDevices(void);

/*
 * This will get a mask of the available input devices on the 
 * machine.
 */
DveLocalAudioInputDeviceMask 
IS_AudioGetAvailInputDevices(void);

/*
 * This will set the input device.
 */ 
int 
IS_AudioSetInputDevice(DveLocalAudioInputDeviceTag device);

/*
 * This will set the output devices for playback.
 */ 
int 
IS_AudioSetOutputDevice(DveLocalAudioOutputDeviceMask device);

/*
 * This sets the output device...single device...
 */
int
IS_AudioSetOutputDeviceSingle(DveLocalAudioOutputDeviceTag device);

/*
 * This will set the audio output monitor to on or off.
 */
int
IS_AudioSetMonitor(int active);

/*
 * Set volume on a 1 to 100 scale.
 */
int 
IS_AudioSetPlayVol(int vol);

/*
 * This mutes the playback operations on the device.
 */
int 
IS_AudioSetPlayMute(int mute);

/*
 * This sets the audio record volume 
 */
int 
IS_AudioSetRecordVol(int vol);

/*
 * Read Audio Data...
 */
int
IS_AudioReadData(char *buffer, int readSize, int doNonBlockingRead);

/*
 * Write Audio Data...
 */
int
IS_AudioWriteData(char *buffer, int writeSize);

/*
 * Get the current audio hardware state...
 *  
 *      this returns -1 on failure. 
 */
int
IS_AudioQueryDeviceStatus(dveAudioInfoType *result);

/*
 * On some platforms, notably HP, tells audio hardware to throw away all
 * remaining audio data when the audio connection is closed.
 * destroy : If True, indicates that all audio data must be discarded on
 *           close.
 */
int IS_AudioSetDestroyOnClose(int destroy);

/*
 * alaw2ulaw
 *     Convert from alaw to mulaw
 *
 * Input : aval : byte to be converted
 * Output :
 * Return : converted mulaw byte
 * Notes :
 */
unsigned char alaw2ulaw(unsigned char aval);

/*
 * ulaw2alaw
 *    Convert from mulaw to alaw
 *
 * Input : uval : mulaw byte to be converted
 * Output :
 * Return : converted alaw byte
 * Notes :
 */
unsigned char ulaw2alaw(unsigned char uval);

#ifdef __cplusplus
}
#endif

#endif /*_INSOFT_AUDIO_UTILS_H_*/
