#ifndef _DVE_AV_H320_H_
#define _DVE_AV_H320_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/AVCtl/core.h>

/* H320 Call setup and handling for C! */

/*
 * dveAVAcceptIncomingCall
 *   Accept an incoming call. This is issued when the application asks for
 *   and receives an incoming call event. If the application does not 
 *   register an event handler for this event, all incoming calls will
 *   be automatically rejected.
 *
 * Input : session : The session returned by dveAVOpenSession()
 * Output : index : On success, this calls adds a new member that represents 
 *                  the remote caller and returns the member index 
 *                  corresponding to this member. This index can now be used 
 *                  in all member-related calls such as disable, enable, etc. 
 *                  Of course, changing the transport info for the member in 
 *                  any way will tear down the call. 
 *
 * Return : dveAVOK on success, appropriate error status otherwise
 *
 * Notes :
 */
DveAVError dveAVAcceptIncomingCall(DveAVSession session, int *index);

/*
 * dveAVRejectIncomingCall
 *    Reject an incoming call. This is issued when the application asks for
 *    and receives an incoming call event. If the application does not 
 *    register an event handler for this event, all incoming calls will
 *    be automatically rejected.
 *
 * Input : session : The session returned by dveAVOpenSession()
 * Output :
 * Return : dveAVOK on success, appropriate error status otherwise
 * Notes :
 */
DveAVError dveAVRejectIncomingCall(DveAVSession session);


/*
 * dveAVTransferIncomingCall
 *     Transfer an incoming call to another endpoint. This is issued when the
 *     application asks for and receives an incoming call event. If the 
 *     application does not register an event handler for this event, all
 *     incoming calls will be automatically rejected.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         t : Transport information for endpoint to transfer the call to
 * Output :
 * Return : dveAVOK on success, appropriate error status otherwise
 * Notes : t should neccessarily be a H.320 address
 */
DveAVError dveAVTransferIncomingCall(DveAVSession, DveAVTransport t);

#ifdef __cplusplus
} 
#endif

#endif /* _DVE_AV_H320_H_ */

