#ifndef _DVE_AV_VIEWER_CONTROL_H_
#define _DVE_AV_VIEWER_CONTROL_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/AVCtl/AVCtl.h>

typedef enum {
  dveAVVrModeInvalid=0,
  dveAVVrModeObsolete,   /* Use old-style C! viewers */
  dveAVVrModeOneOnOne,   /* Assign each video stream to a new viewer 
			  * executable. The Engine will always auto-start
			  * the viewer executable(s) when necessary. The 
			  * application cannot pass the viewer any command-
			  * line arguments
			  */
  dveAVVrModeAllOnOne,   /* Assign all video streams to one viewer 
			  * executable, auto-starting the viewer when
			  * necessary. The application cannot pass the 
			  * viewer command-line arguments
			  */
  dveAVVrModeCustom      /* The application writes and runs its own viewers;
			  * the Engine will not auto-start viewers, it is the
			  * application's responsibility to start viewers
			  * and assign members to viewers
			  */
  } DveAVVrMode;


/*
 * dveAVSetViewerMode
 *   Define the mode in which the viewer will run. See above for various
 *   modes; the default is dveAVVrModeObsolete. 
 * 
 * Input : mode : New mode
 *         viewerPath : Path Name of viewer to auto-start in OneOnOne and
 *                      AllOnOne modes. Ignored otherwise.
 * 
 * Output :
 * Return : dveAVOK on success, appropriate error status otherwise.
 * Notes :
 */
DveAVError dveAVSetViewerMode(DveAVSession session, DveAVVrMode mode,
			      char *viewerPath);


/*
 * dveAVStartViewer
 *    Create a new cluster. 
 * 
 * Input : session : The session returned by dveAVOpenSession()
 * Output :
 * Return : A positive integer called the viewer id that will be henceforth
 *          used to identify this viewer, -1 on error
 * Notes : This function will block until the viewer is ready and initialized.
 *         However, your OpenDVE/AV callbacks may still be called in the
 *         meantime. Also, this function will return an error if called
 *         in any viewer mode other than dveAVVrModeCustom.
 */
int dveAVStartViewer(DveAVSession session, char *viewerPath, int argc, 
		     char *argv[]);

/*
 * dveAVAddMemberToViewer
 *     Add a member to a viewer. A member can only be assigned to one 
 *     viewer at a time, so this will automatically remove this member from 
 *     any other viewer that the member is already assigned to.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         viewerId : The id returned by dveAVStartViewer()
 *         memberIndex : Member index of the member
 * Output :
 * Return : dveAVOK on success, appropriate error status otherwise
 * Notes : This function will return an error if called in any viewer mode
 *         other than dveAVVrModeCustom.
 */
DveAVError dveAVAddMemberToViewer(DveAVSession session, int viewerId,
				  int memberIndex);

/*
 * dveAVStopViewer
 *   Stop (kill) an already running viewer. A viewer should always be stopped
 *   using this call. This gives the viewer and the Engine a chance to clean 
 *   up gracefully.
 * 
 * Input : 
 * Output :
 * Return : dveAVOK on success, appropriate error status otherwise
 * Notes : This function will return an error if called in any viewer mode
 *         other than dveAVVrModeCustom.
 */
DveAVError dveAVStopViewer(DveAVSession session, int viewerId);

/*
 * dveAVSendMsgToViewer
 *   This is a function provided for the application's convenience. 
 *   The application using the AV API and the viewer may be separate
 *   processes (although they could be the same process). In such cases,
 *   this function provides a ready-made channel of communication between the
 *   controlling application and the viewer. This function allows the 
 *   controlling application to send a custom message to a viewer. 
 * 
 * Input : viewerId : ID of viewer
 *         msg : Data to be sent to viewer
 *         size : Size of data
 * Output :
 * Return :
 * Notes :
 */
DveAVError dveAVSendMsgToViewer(DveAVSession session, int viewerId,
				void *msg, int size);


#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_VIEWER_CONTROL_H_ */




