/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the API for plugin-to-plugin connections.
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Includes:
 *
 * Implemented by:
 *  lib/dveConns (plugin-side)
 *
 * Types:
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 27 March 1995
 *
 * Known Bugs:
 *
 * Notes:
 *
 * Change Log:
 *  $Log$
 */
#ifndef _OPENDVE_PLUGIN_TO_PLUGIN_CONNECTIONS_API_H_
#define _OPENDVE_PLUGIN_TO_PLUGIN_CONNECTIONS_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

typedef struct _DveConn* DveConn;

typedef enum {
  dveConnStateInvalid=0,
  dveConnStatePending,
  dveConnStateOpened,
  dveConnStateClosed,
  dveConnStateDead /* The user is no longer in conference */
} DveConnState;

typedef void (*DveConnStateCallback)(DveConn dveConn,
				     DveConnState state,
				     void* context);

/* Opens a connection to a plugin.  A connection has three possible states:
 *  pending (we've sent a message to the connection manager, and we're
 *  waiting for a reply), opened (the connection manager has said, "yes, it's
 *  here"), or closed (the connection manager has said, "it's gone away now").
 *  The callback will be invoked whenever the state changes.  Accordingly (and
 *  unusually), the callback is necessary.
 *
 * The initial state is pending.  The only possible state transitions are
 *  pending to opened, opened to closed, closed to opened (which last
 *  occurs if the plugin returns), and anything to dead.  When the state
 *  transitions to dead, you will get a callback; upon return from the
 *  callback, the DveConn will be closed.
 */
DveConn dveConnOpen(DveSession dveSession,
		    DveUserHandle handle,
		    DvePort port,
		    DveConnStateCallback stateCallback,
		    void* context);

/* Close the connection.  There is no problem doing this from inside a state
 *  callback.
 */
void dveConnClose(DveConn dveConn);

/* Get information on the connection. */
DveUserHandle dveConnHandle(DveConn dveConn);
DveSession dveConnSession(DveConn dveConn);
DvePort dveConnPort(DveConn dveConn);
DveConnState dveConnState(DveConn dveConn);

#ifdef __cplusplus
}
#endif

#endif /*_OPENDVE_PLUGIN_TO_PLUGIN_CONNECTIONS_API_H_*/
