/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the events for OpenDVE
 *
 * Requires:
 *  <dve/low.h>
 *
 * Includes:
 *  <dve/directLow.h>
 *
 * Implemented by:
 *  dve.c
 *
 * Types:
 *  DveEventTag
 *  DveEventMask
 *  DveEventStateArg
 *  DveEventStartingArg
 *  DveEventNewMemberArg
 *  DveEventMemberRejectedArg
 *  DveEventMemberLeftArg
 *  DveEventLocalMemberInfoArg
 *  DveEventDNDChangedArg
 *  DveEventNewDirectArg
 *  DveEventClosedDirectArg
 *  DveEventAcceptedDirectArg
 *  DveEventRejectedDirectArg
 *  DveEventStartedDirectArg
 *  DveEventArg
 *  DveEvent
 *
 * Author:
 *  francis, ?? Feb 1994
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_EVENTS_H_
#define _DVE_EVENTS_H_

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/directLow.h>

typedef enum
{
  dveEventTagInvalid=0,

  /* plugin-to-engine protocol events--these have no arguments */
  dveEventTagEngineGreeting,
  dveEventTagFirst=dveEventTagEngineGreeting,
  dveEventTagFirstEngine=dveEventTagFirst,
  dveEventTagEngineShutdown,
  dveEventTagEngineRejected,
  dveEventTagEngineClosed,
  dveEventTagLastEngine=dveEventTagEngineClosed,

  /* conference room events */
  dveEventTagState,
  dveEventTagFirstUser=dveEventTagState,
  dveEventTagStarting,
  dveEventTagNewMember,
  dveEventTagMemberAccepted,
  dveEventTagMemberRejected,
  dveEventTagMemberLeft,
  dveEventTagLocalMemberInfo,
  dveEventTagDNDChanged,
  dveEventTagLastUser=dveEventTagDNDChanged,

  /* direct-connection events */
  dveEventTagDirectNew,
  dveEventTagFirstDirect=dveEventTagDirectNew,
  dveEventTagDirectClosed,
  dveEventTagDirectAccepted,
  dveEventTagDirectRejected,
  dveEventTagDirectStarted,
  dveEventTagLastDirect=dveEventTagDirectStarted,

  dveEventTagConnected,
  dveEventTagLast=dveEventTagConnected
  } DveEventTag;

typedef enum
{
  dveEventMaskNull=0,

  dveEventMaskEngineGreeting=(1<<dveEventTagEngineGreeting),
  dveEventMaskEngineShutdown=(1<<dveEventTagEngineShutdown),
  dveEventMaskEngineRejected=(1<<dveEventTagEngineRejected),
  dveEventMaskEngineClosed=(1<<dveEventTagEngineClosed),
  dveEventMaskAllEngine=(  dveEventMaskEngineGreeting
			 | dveEventMaskEngineShutdown
			 | dveEventMaskEngineRejected
			 | dveEventMaskEngineClosed
			 ),

  dveEventMaskState=(1<<dveEventTagState),
  dveEventMaskStarting=(1<<dveEventTagStarting),
  dveEventMaskNewMember=(1<<dveEventTagNewMember),
  dveEventMaskMemberAccepted=(1<<dveEventTagMemberAccepted),
  dveEventMaskMemberRejected=(1<<dveEventTagMemberRejected),
  dveEventMaskMemberLeft=(1<<dveEventTagMemberLeft),
  dveEventMaskLocalMemberInfo=(1<<dveEventTagLocalMemberInfo),
  dveEventMaskDNDChanged=(1<<dveEventTagDNDChanged),
  dveEventMaskConnected=(1<<dveEventTagConnected),
  dveEventMaskAllUser=(  dveEventMaskState
		       | dveEventMaskStarting
		       | dveEventMaskNewMember
		       | dveEventMaskMemberAccepted
		       | dveEventMaskMemberRejected
		       | dveEventMaskMemberLeft
		       | dveEventMaskLocalMemberInfo
		       | dveEventMaskDNDChanged
		       | dveEventMaskConnected
		       ),
  dveEventMaskAll=(dveEventMaskAllEngine | dveEventMaskAllUser)
} DveEventMask;

typedef struct {
  DvePort actPort;
} DveEventEngineGreetingArg;

typedef struct {
  ConfdStateTag state;
} DveEventStateArg;

typedef struct {
  DveConfParticipant part;
  Bool16 active,align;
} DveEventStartingArg_Member;

typedef struct {
  int numMembers,self; /* self is index into members for myself */
  Bool16 DND,align1;
  ConfdStateTag state;
  char align2[ALIGNSIZE(ConfdStateTag)];
  DveIPAList addrs;
  DveEventStartingArg_Member members[1];
} DveEventStartingArg;

typedef struct {
  DveConfParticipant member;
} DveEventNewMemberArg,
  DveEventMemberAcceptedArg,
  DveEventMemberLeftArg;

typedef struct {
  DveConfParticipant member;
  u_int32 reason; /* Actually ConfdRejectionReason (see low.h) */
} DveEventMemberRejectedArg;

typedef struct {
  DveConfParticipantLong member;
  Bool16 DND;
} DveEventLocalMemberInfoArg;

typedef struct {
  Bool16 DND;
} DveEventDNDChangedArg;

typedef struct {
  DveUserHandle userHandle;
  DveIPA addr;
  DveIPAList addrs;
} DveEventConnectedArg;

typedef struct {
  DveDirectID direct;
} DveEventNewDirectArg,
  DveEventClosedDirectArg,
  DveEventAcceptedDirectArg,
  DveEventRejectedDirectArg,
  DveEventStartedDirectArg;

typedef union {
  DveEventEngineGreetingArg engineGreeting;
  DveEventStateArg state;
  DveEventStartingArg starting;
  DveEventNewMemberArg newMember;
  DveEventMemberAcceptedArg memberAccepted;
  DveEventMemberRejectedArg memberRejected;
  DveEventMemberLeftArg memberLeft;
  DveEventLocalMemberInfoArg localMemberInfo;
  DveEventDNDChangedArg DNDChanged;
  DveEventConnectedArg connected;
  
  DveEventNewDirectArg newDirect;
  DveEventClosedDirectArg closedDirect;
  DveEventAcceptedDirectArg acceptedDirect;
  DveEventRejectedDirectArg rejectedDirect;
  DveEventStartedDirectArg startedDirect;
} DveEventArg;

typedef struct {
  u_int32 size;
  u_int32 reqID, evtID;
  DveEventTag tag;
  DveEventArg arg;
} DveEvent;

#ifdef __cplusplus
}
#endif

#endif /*_DVE_EVENTS_H_*/
