#ifndef _DVE_PLATFORM_INFO_H_
#define _DVE_PLATFORM_INFO_H_

#ifdef __cplusplus
extern "C" {
#endif

#define dvePlatformClipName "platforms"

#ifndef _DVE_PLATFORM_INFO_NO_FUNCTIONS_
static const char* dvePlatformClipFormat(DveUserHandle handle)
{
  static char format[16];
  sprintf(format,"platform:%d",(int)handle);
  return format;
}

static DveUserHandle dvePlatformHandle(char* format)
{
  return (DveUserHandle)atoi(format+9);
}

typedef enum {
  dvePlatformHardwareTagInvalid=0,
  dvePlatformHardwareTagIntelPC,
  dvePlatformHardwareTagHP9000,
  dvePlatformHardwareTagSparc,
  dvePlatformHardwareTagRS6000,
  dvePlatformHardwareTagAXP,
  dvePlatformHardwareTagSGI_MIPS
} DvePlatformHardwareTag;

typedef enum {
  dvePlatformOSTagInvalid=0,
  dvePlatformOSTagWin16,
  dvePlatformOSTagWin32,
  dvePlatformOSTagSunOS,
  dvePlatformOSTagSolaris,
  dvePlatformOSTagHPUX9,
  dvePlatformOSTagHPUX10,
  dvePlatformOSTagAIX,
  dvePlatformOSTagOSF1V1,
  dvePlatformOSTagOSF1V2,
  dvePlatformOSTagIRIX,
  dvePlatformOSTagWinChicago=21, /* Chicago was the old code name */
  dvePlatformOSTagWin95=dvePlatformOSTagWinChicago,
  dvePlatformOSTagWin32s=22
} DvePlatformOSTag;

typedef struct {
  DvePlatformHardwareTag hardware;
  DvePlatformOSTag os;
} DvePlatform;

typedef enum {
  dvePlatformFeatureFlagNull=0,
  dvePlatformFeatureFlagPing=1,
  dvePlatformFeatureFlagClip=2,
  dvePlatformFeatureFlagMutex=4,
  dvePlatformFeatureFlagReasons=8,
  dvePlatformFeatureFlagPluginConns=16
  } DvePlatformFeatureFlags;

/* This is the data structure filed under "platform:<userHandle>" */
typedef struct {
  DveVersion appCapsVersion;
  DvePlatform platform,realPlatform;
  DveVersion appVersion;      /* C! version                             */
  DvePlatformFeatureFlags featureFlags;
  u_int16 maxParts;           /* maximum number of the participants     */
  u_int16 maxUdpDg;           /* max size of UDP packet (system-dep)    */
} DvePlatformAppCaps;

#endif /*_DVE_PLATFORM_INFO_NO_FUNCTIONS_*/

#ifdef _DVE_MAIN_API_H_
DvePlatformAppCaps* dvePlatformGetAppCaps(DveSession dveSession,
					  DveUserHandle handle);
const char* dvePlatformHardwareName(DvePlatformHardwareTag tag);
const char* dvePlatformOSName(DvePlatformOSTag tag);

int dvePlatformMaxHardwareNameLen(void);
int dvePlatformMaxOSNameLen(void);

/* Returns 0 if the remote user does not do connections, 1 if they
 *  say they do, and 2 if they don't say, but I think they do (i.e,
 *  they're C! 4.1, and not a PC).  Returns -1 on error.
 */
int dvePlatformDoesConnections(DveSession dveSession,
			       DveUserHandle handle);
#endif /*_DVE_MAIN_API_H_*/

#ifdef __cplusplus
}
#endif

#endif /*_DVE_PLATFORM_INFO_H_*/
