/*
//Required
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the service-registry library, which provides (a) convenience
 *   functions for registering with a sharable plugin such as a help viewer
 *   and (b) functions for implementing a sharable plugin.
 *
 * Requires:
 *  <dve/dve.h>
 *
 * Includes:
 *  <dve/conns.h>
 *  <dve/pluginManagment.h>
 *
 * Implemented by:
 *  lib/dveSvcRegistry/
 *
 * Types:
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
 * Author:
 *  francis, 1 May 1995
 *
 * Known Bugs:
 *
 * Notes:
 *
 * Change Log:
 *  $Log$
 */
#ifndef _OPENDVE_PLUGIN_SERVICE_REGISTRY_API_H_
#define _OPENDVE_PLUGIN_SERVICE_REGISTRY_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <dve/conns.h>
#include <dve/pluginManagement.h>

typedef struct _DveSvcClient* DveSvcClient;

typedef enum {
  dveSvcClientCallbackArgInvalid=0,
  dveSvcClientCallbackArgPresent,
  dveSvcClientCallbackArgGone,
  dveSvcClientCallbackArgAck,
  dveSvcClientCallbackArgNak
} DveSvcClientCallbackArg;

/* If arg is Gone or Nak, the client will be deleted on return from the
 *  callback.
 */
typedef void (*DveSvcClientCallback)(DveSvcClient client,
				     DveSvcClientCallbackArg arg,
				     void* context);

/* DveSvcClient with the given server (sharable plugin); a plugin-to-plugin
 *  connection will be opened to monitor whether the server is running.  The
 *  callback will be invoked when the server goes away, is found to be
 *  running, or replies (with RegisterAck or RegisterNak); the context will
 *  be passed to the callback.
 */
DveSvcClient dveSvcRegisterClient(DveSession dveSession,
				  DvePort server,
				  DveSvcClientCallback callback,
				  void* context);
/* Unregister with the server. */
void dveSvcUnregisterClient(DveSvcClient client);

/* Sends a Quit message to the server, which may or may not stop it. */
int dveSvcStopServer(DveSession dveSession,
		     DvePort server);

DveSession dveSvcGetSession(DveSvcClient client);
DvePort dveSvcGetServer(DveSvcClient client);
void* dveSvcGetContext(DveSvcClient client);

/* Server-side stuff.  Note that the server's user callback is set, not
 *  added, so that it cannot coexist with older plugin-management stuff.  This
 *  is because the old stuff's Quit handler would override the proper register/
 *  unregister behavior on the server.  Make sure the older stuff doesn't
 *  initialize after the server is opened--that would replace the callback.
 */
typedef struct _DveSvcServer* DveSvcServer;

/* Callback invoked on Hide, Show, or (successful) Quit */
typedef void (*DveSvcServerCallback)(DveSvcServer server);

DveSvcServer dveSvcBeAServer(DveSession dveSession);
void dveSvcCloseServer(DveSvcServer server);

/* Set a callback; tag must be Hide, Show, or Quit. */
int dveSvcSetServerCallback(DveSvcServer server,
			    DvePluginManagementMsgTag tag,
			    DveSvcServerCallback callback);

#ifdef __cplusplus
}
#endif

#endif /*_OPENDVE_PLUGIN_SERVICE_REGISTRY_API_H_*/
