#ifndef _DVE_XVIEW_RECIPIENT_LIST_H_
#define _DVE_XVIEW_RECIPIENT_LIST_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <xview/xview.h>
#include <xview/panel.h>
#include <dve/dve.h>

typedef struct _DveXViewRecipientList* DveXViewRecipientList;

/* The object should be a scrolled list.  [If it isn't, the function will
 *  return NULL.]  Do not attempt to change the selection policy after
 *  creating the DveXViewRecipientList; it will not be happy.
 */
DveXViewRecipientList dveXViewRecipientListCreate(DveSession dveSession,
						  Xv_opaque list,
						  DvePartList* partList,
						  Bool16 excludeSelf);

typedef enum {
  dveXViewRecipientListChangeTagInvalid=0,
  dveXViewRecipientListChangeTagAdded,
  dveXViewRecipientListChangeTagDeleted,
  dveXViewRecipientListChangeTagChanged,
  dveXViewRecipientListChangeTagAction
} DveXViewRecipientListChangeTag;

typedef struct {
  DveUserHandle member;
} DveXViewRecipientListChangeAddedArg,
  DveXViewRecipientListChangeDeletedArg,
  DveXViewRecipientListChangeChangedArg,
  DveXViewRecipientListChangeActionArg;

typedef union {
  DveXViewRecipientListChangeAddedArg added;
  DveXViewRecipientListChangeDeletedArg deleted;
  DveXViewRecipientListChangeChangedArg changed;
  DveXViewRecipientListChangeActionArg action;
} DveXViewRecipientListChangeArg;

typedef struct {
  DveXViewRecipientListChangeTag tag;
  DveXViewRecipientListChangeArg arg;
} DveXViewRecipientListCallbackStruct;

typedef void (*DveXViewRecipientListCallback)(DveXViewRecipientList list,
					      DveXViewRecipientListCallbackStruct* cbs);

void dveXViewRecipientListSetCallback(DveXViewRecipientList list,
				      DveXViewRecipientListCallback callback);

/* As for dve{S,G}etUserData(), it is your responsibility to manage
 *  the memory pointed to by userData.
 */
void dveXViewRecipientListSetUserData(DveXViewRecipientList list,
				      void* userData);
void* dveXViewRecipientListGetUserData(DveXViewRecipientList list);

/* Gives access to the list of selected members.  Do *not* modify this
 *  list; to change it, use the [de]select functions below.
 */
const DvePartList*
  dveXViewRecipientListGetPartList(DveXViewRecipientList list);

/* Meaningful only in browse mode; in multiple-select mode, returns -1.
 */
int dveXViewRecipientListGetSelection(DveXViewRecipientList list);

/* The following do not invoke the callback */
void dveXViewRecipientListSelect(DveXViewRecipientList list,
				 DveUserHandle member);
void dveXViewRecipientListSelectAll(DveXViewRecipientList list);
void dveXViewRecipientListDeselect(DveXViewRecipientList list,
				   DveUserHandle member);
void dveXViewRecipientListDeselectAll(DveXViewRecipientList list);
void dveXViewRecipientListSelectThese(DveXViewRecipientList list,
				      DvePartList* members,
				      Bool16 giveCallbacks);
void dveXViewRecipientListDeselectThese(DveXViewRecipientList list,
					DvePartList* members,
					Bool16 giveCallbacks);
void dveXViewRecipientListSetSelection(DveXViewRecipientList list,
				       DvePartList* members,
				       Bool16 giveCallbacks);

/* Call this when an OpenDVE event changes the set of active
 * members.
 */
void dveXViewRecipientListHandleDveEvent(DveXViewRecipientList list,
					 DveEvent* event);

/* If autoSelect is set to True, members will be selected as soon as
 *  they accept (when the conference starts, all members will be
 *  selected).  The default is False.
 *
 * Clearly, autoSelect is False when the list is first created.  If
 *  it is set to True before any X or OpenDVE events are processed
 *  for the list (immediately after creation should do), all
 *  conference members will be selected.
 *
 * autoSelect is ignored for browse-mode lists.
 */
void dveXViewRecipientListSetAutoSelect(DveXViewRecipientList list,
					Bool16 autoSelect);
Bool16 dveXViewRecipientListGetAutoSelect(DveXViewRecipientList list);

void dveXViewRecipientListCacheSelections(DveXViewRecipientList list);
void dveXViewRecipientListRestoreSelections(DveXViewRecipientList list);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_XVIEW_RECIPIENT_LIST_H_*/
