/*
 * Name:        Plugin_procs.c
 *
 * Description:
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <Xm/Xm.h>

#include <stdio.h>
#include <sys/types.h>

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <X11/Xutil.h>

#include <dve/dve.h>
#include <dve/clipboard.h>

#include "mpApplicationShell.h"
#include "ISLogoPluginProtocol.h"

#include "InSoft_64"

Pixmap create_logo();

void redisplay();
void resize();
void HidePlugin(void);
void ShowPlugin(void);

extern DveSession dveSession;
extern DveClipboard clipboard;

extern Widget mpApplicationShell;
extern Widget mpFooterLabel;
extern Widget drawingArea1;

static const DvePort portNum=LOGO_PLUGIN_PORT;
static XmTextPosition wpr_position;

typedef struct {
   Pixmap pix;
   GC gc;
   Dimension width, height;
} pixmap_data;

/*
 * Insert the text string into our footer.
 */
void
mpSetFooter(const char *msg)
{
   XmString str=XmStringCreateSimple((char*)msg);
   XtVaSetValues(mpFooterLabel,
                 XmNlabelString,str,
                 NULL);
   XmStringFree(str);
}

/*
 * Open the DVE clipboard and get started.
 */
void dveAppInit(void)
{
   XGCValues values;
   Arg wargs[10];
   static pixmap_data data;
   int n;

   /* 
    * Establish OpenDVE Clipboard.
    */
   if (InitializePluginClipboard() < 0){
      fprintf(stderr, "Initialization of OpenDVE Clipboard Failed.\n\7");
   }

   /*
    * Use the foreground and background colors of the drawingArea to
    * create a graphics context.
    */
   n = 0;
   XtSetArg(wargs[n], XtNforeground, &values.foreground); n++;
   XtSetArg(wargs[n], XtNbackground, &values.background); n++;
   XtGetValues(drawingArea1, wargs, n);
   data.gc = XtGetGC(drawingArea1, GCForeground | GCBackground, &values);
   
   /*
    * Create the Pixmap containing the InSoft logo.  Store the
    * Pixmap, as well as the size of the Pixmap in the struct.
    */
   data.width = InSoft_64_width;
   data.height = InSoft_64_height;
   data.pix = create_logo(drawingArea1, data.gc, InSoft_64_bits, InSoft_64_width, InSoft_64_height);
 
   XtAddCallback(drawingArea1, XmNexposeCallback, redisplay, &data);
   XtAddCallback(drawingArea1, XmNresizeCallback, resize, &data);
}

/*
 * Capture the image from the display, convert the image from 
 * XImage to DIB, and then copy the DIB formatted data to the
 * OpenDVEClipboard.
 */
void
Copy_b_Callback(void)
{
   Display *disp;
   Window win;
   XImage* xim;
   Colormap cmap;
   char* dibdata;
   int screen;
   int dibsize;

   Dimension widget_width, widget_height;
   Arg wargs[10];
   int n;
 
   /*
    * Grab a pointer to the Xlib XDisplay structure used
    * by the widget.
    */
   disp = XtDisplay(drawingArea1);
 
   /*
    * Retrieve the ID of the window used by the widget.
    */
   win = XtWindow(drawingArea1);
 
   n = 0;
   XtSetArg(wargs[n], XtNwidth, &widget_width); n++;
   XtSetArg(wargs[n], XtNheight, &widget_height); n++;
   XtGetValues(drawingArea1, wargs, n);

   /*
    * Place the contents of the defined rectangle from our drawable
    * into an XImage.
    */
   xim = XGetImage(disp, win, 0, 0, widget_width, widget_height, AllPlanes, ZPixmap);
 
   screen = DefaultScreen(disp);
   cmap = DefaultColormap(disp, screen);
 
   /*
    * This function is provided by InSoft's library
    * entitled dve_Imaging.a.  The OpenDVE clipboard exchanges
    * data in a DIB (Device Independent Bitmap) format.
    * dveXImageToDIB() provides developers with a method
    * to get their data into this format.
    */
   if (dveXImageToDIB(disp, xim, cmap, &dibsize, &dibdata) < 0)
   {
      mpSetFooter("Error converting to DIB format.");
      return;
   }

   /*
    * Place this image in the clipboard.
    */
   if (CopyDataToClipboard(clipboard, dibdata, dibsize, IS_IMAGE_CLIPBOARD_ITEM) < 0)
   {
      mpSetFooter("Error copying to Clipboard.");
   }
   else
   {
      mpSetFooter("Copied to Clipboard.");
   }
}

/*
 * Shutdown the Open DVEClipboard, close our session with the
 * conference engine, and unregister with Xt.
 */
void
Exit_b_Callback(void)
{
   HidePlugin();
}

/*
 * Request to redisplay.  Determine the width and height of 
 * drawingArea1 and then center the pixmap_data in this
 * area.
 */
void redisplay(Widget w, pixmap_data *data,
               XmDrawingAreaCallbackStruct *call_data)
{
   Dimension widget_width, widget_height;
   Arg wargs[10];
   int n;

   /*
    * Get the current size of the widget window.
    */
   n = 0;
   XtSetArg(wargs[n], XtNwidth, &widget_width); n++;
   XtSetArg(wargs[n], XtNheight, &widget_height); n++;
   XtGetValues(w, wargs, n);

   /*
    * Copy the contents of the pixmap to the center of the window.
    */
   XCopyPlane(XtDisplay(w), data->pix, XtWindow(w), data->gc,
             0, 0, data->width, data->height,
             (int)(widget_width - data->width) / (int)2,
             (int)(widget_height - data->height) / (int)2, 1);

   mpSetFooter(" ");
}

/*
 * A resize event has occurred.  Clear the drawingArea before
 * redisplaying the image.
 */
void resize(Widget w, caddr_t data, XmDrawingAreaCallbackStruct *call_data)
{
   if (XtIsRealized(w))
   {
      XClearArea(XtDisplay(w), XtWindow(w), 0, 0, 0, 0, TRUE);
      mpSetFooter(" ");
   }
}

/*
 * Create a bitmap to be placed in the pixmap_data structure.
 */
Pixmap create_logo(Widget w, GC gc, char *bits, int width, int height)
{
   Pixmap bitmap;
 
   /*
    * Create a bitmap from the data.
    */
   bitmap = XCreateBitmapFromData(XtDisplay(w),
                                  RootWindowOfScreen(XtScreen(w)),
                                  bits, width, height);
 
   return bitmap;
}

void
introduceYourself(void)
{
   /*
    * Start up OpenDVE.
    */
   if (InitializeOpenDVE(UxAppContext, LOGO_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }
}

void
HidePlugin(void)
{
   XtPopdown(mpApplicationShell);
}
 
void
ShowPlugin(void)
{
   XtPopup(mpApplicationShell, XtGrabNone);
}
