/*
 *
 * Name:        ISSamplePluginOL.c
 *
 * Description: Provide main routine for ISSamplePluginOL
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <group.h>

#include "main_ui.h"

#include <dve/dve.h>
#include "SamplePluginProtocol.h"

/*
 * External variable declarations.
 */
main_window1_objects	*Main_window1;

#ifdef MAIN

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;

char *buf;

/*
 * Notify interpose destroy function.
 */
Notify_value
QuitInterposer(Frame frame, Destroy_status status)
{
   int result;

   switch (status) {
      case DESTROY_CHECKING:
         HidePlugin();
         notify_veto_destroy(frame);
         return(NOTIFY_DONE);
      case DESTROY_CLEANUP:
         return(NOTIFY_DONE);
      case DESTROY_PROCESS_DEATH:
         return(notify_next_destroy_func(frame, status));
         break;
   }
}

/*
 * Handle general signals gracefully and exit.
 */
Notify_value
SignalHandlingFunction(Notify_client client, int sig)
{
   if (sig != SIGQUIT)
      fprintf(stderr, "\7SIGNAL %d Encountered...Exiting.\n", sig);
   CleanupPlugin();
   exit(0); 
}

/*
 * Main()
 */
void
main(int argc, char *argv[])
{
   /*
    * Initialize XView.
    */
   xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
   INSTANCE = xv_unique_key();
	
   /*
    * Initialize user interface components.
    * Do NOT edit the object initializations by hand.
    */
   Main_window1 = main_window1_objects_initialize(NULL, NULL);
	
   /*
    * Start up OpenDVE.
    */
   if (InitializeOpenDVE(Main_window1->window1, SAMPLE_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

   /*
    * Create a DveClipboard.
    */
   if (InitializePluginClipboard() < 0)
   {
      fprintf(stderr, "Initialization of OpenDVE Clipboard Failed.\n\7");
   }

   /*
    * Catch exits from plugin.
    */
   (void) notify_interpose_destroy_func(Main_window1->window1, QuitInterposer);
   (void) notify_set_signal_func(Main_window1->window1,
                                 SignalHandlingFunction, SIGQUIT, NOTIFY_ASYNC);
   (void) notify_set_signal_func(Main_window1->window1,
                                 SignalHandlingFunction, SIGINT, NOTIFY_ASYNC);
   (void) notify_set_signal_func(Main_window1->window1,
                                 SignalHandlingFunction, SIGSEGV, NOTIFY_ASYNC);
   (void) notify_set_signal_func(Main_window1->window1,
                                 SignalHandlingFunction, SIGBUS, NOTIFY_ASYNC);

   /*
    * Should we start this plugin mapped?
    */
   if (dveShouldStartMapped(&argc,argv))
      ShowPlugin();

   /*
    * Turn control over to XView...with notify_start() NOT xv_main_loop()
    * which will automatically map the Plugin on startup.
    */
   notify_start();
   exit(0);
}

#endif
