/*
 * Name:        main_stubs.c
 *
 * Description: Provides panel item callback routines for ISSamplePluginOL
 *
 * Change Log:
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <group.h>

#include "ISSamplePluginOL.h"

#include <dve/dve.h>
#include <dve/clipboard.h>
#include "SamplePluginProtocol.h"
#include "main_ui.h"


#ifdef MAIN


#endif

/* 
 * External definitions established in OpenDVE_procs.c
 */
extern DveSession dveSession;
extern DveClipboard clipboard;

/*
 * Global object definitions.
 */
main_window1_objects	*Main_window1;

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;

/*
 * Notify callback function for `slider1'.
 */
void
NotifyTemperature(Panel_item item, int value, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
   /* 
    * Send the new temperature setting to all conference participants.
    */
   SendCurrentStatus(TEMPERATURE_CHANGE);

	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

}

/*
 * Notify callback function for `setting2'.
 */
void
NotifyMusic(Panel_item item, int value, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
   short i;
	
   for (i = 0; i < 3; i++) {
      if (value & 01)
		
		/* gxv_start_connections DO NOT EDIT THIS SECTION */

		/* gxv_end_connections */

      value >>= 1;
   }
   SendCurrentStatus(MUSIC_CHANGE);
}

/*
 * Notify callback function for `setting1'.
 */
void
NotifyDrink(Panel_item item, int value, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
   SendCurrentStatus(DRINK_CHANGE);

	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

}

/*
 * Notify callback function for `copybutton'.
 */
void
NotifyCopy(Panel_item item, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
   /* 
    * Read the text string from the sendmessage panel item.
    */
   char *data = (char *)xv_get(Main_window1->sendmessage, PANEL_VALUE);
   if (data[0] == '\0')
   {
      XBell((char *)xv_get(Main_window1->window1, XV_DISPLAY), (int)0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Enter a send message to copy.", NULL);
      return;
   }

   /*
    * Copy the string into the established local DveClipboard.
    */
   if (CopyDataToClipboard(clipboard, data, strlen(data), 
                           IS_TEXT_CLIPBOARD_ITEM) < 0)
   {
      XBell(xv_get(Main_window1->window1, XV_DISPLAY), 0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Error Copying Clipboard.", NULL);
   }
   else
   {
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Copy Complete.", NULL);
   }
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

}

/*
 * Notify callback function for `showbutton'.
 */
void
NotifyShow(Panel_item item, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
   /* 
    * Obtain clipboard contents.  In this case, our DveClipboard
    * contents are of type, IS_TEXT_CLIPBOARD_ITEM, defined as "text."
    */
   if (GetOpenDVEClipboard(clipboard, IS_TEXT_CLIPBOARD_ITEM) < 0)
   {
      textsw_insert(Main_window1->textpane, "==========\n", 11);
      textsw_insert(Main_window1->textpane, "Clipboard Empty!\n",17);
      return;
   }

	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

}

/*
 * Notify callback function for `sendbutton'.
 */
void
NotifySend(Panel_item item, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
   char *data = (char *)xv_get(Main_window1->sendmessage, PANEL_VALUE);
   if (data[0] == '\0')
   {
      XBell(xv_get(Main_window1->window1, XV_DISPLAY), 0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Enter a send message.", NULL);
      return;
   }
		
   /*
    * The sendButton has been selected.  Read the contents of the
    * sendmessage item and pass them to SendTextMessage().
    * SendTextMessage() in turn fills the message component
    * of the samplePluginMessageStructure and then uses SendPluginMessage()
    * to notify other conference participants.
    */
   if (SendTextMessage(data, strlen(data)) < 0)
   {
      XBell(xv_get(Main_window1->window1, XV_DISPLAY), 0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Error Sending Message.", NULL);
   }
   else
   {
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Message Sent.", NULL);
      xv_set(Main_window1->sendmessage, PANEL_VALUE, "", NULL);
   }
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

}

/*
 * Notify callback function for `sendmessage'.
 */
Panel_setting
NotifySendMessage(Panel_item item, Event *event)
{
   main_window1_objects *ip = (main_window1_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
   char *	value = (char *) xv_get(item, PANEL_VALUE);
	
   if (value[0] == '\0')
   {
      XBell(xv_get(Main_window1->window1, XV_DISPLAY), 0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Enter a send message.", NULL);
      return;
   }
		
   if (SendTextMessage(value, strlen(value)) < 0)
   {
      XBell(xv_get(Main_window1->window1, XV_DISPLAY), 0);
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Error Sending Message.", NULL);
   }
   else
   {
      xv_set(Main_window1->window1, FRAME_LEFT_FOOTER, 
                                    "Message Sent.", NULL);
      xv_set(Main_window1->sendmessage, PANEL_VALUE, "", NULL);
   }
	
	/* gxv_start_connections DO NOT EDIT THIS SECTION */

	/* gxv_end_connections */

   return panel_text_notify(item, event);
}
