#ifndef _DVE_MOTIF_RECIPIENT_LIST_H_
#define _DVE_MOTIF_RECIPIENT_LIST_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/List.h>
#include <dve/dve.h>

typedef struct _DveMotifRecipientList* DveMotifRecipientList;

/* The Widget should be an XmList.  If it isn't, the function will return
 *  NULL.  Also, only browse- and multiple-select policies are supported
 *  at this time; if the policy is single- or extended-select, the function
 *  will return NULL.  (The single- and extended-select policies will likely
 *  never be supported, as they complicate the issue for the user--when you
 *  see a list, how does it behave?--and are a bad idea.)  Do not attempt to
 *  change the selection policy after creating the DveMotifRecipientList; it
 *  will not be happy.
 */
DveMotifRecipientList dveMotifRecipientListCreate(DveSession dveSession,
						  Widget list,
						  DvePartList* partList,
						  Boolean excludeSelf);

typedef enum {
  dveMotifRecipientListChangeTagInvalid=0,
  dveMotifRecipientListChangeTagAdded,
  dveMotifRecipientListChangeTagDeleted,
  dveMotifRecipientListChangeTagChanged,
  dveMotifRecipientListChangeTagAction
} DveMotifRecipientListChangeTag;

typedef struct {
  DveUserHandle member;
} DveMotifRecipientListChangeAddedArg,
  DveMotifRecipientListChangeDeletedArg,
  DveMotifRecipientListChangeChangedArg,
  DveMotifRecipientListChangeActionArg;

typedef union {
  DveMotifRecipientListChangeAddedArg added;
  DveMotifRecipientListChangeDeletedArg deleted;
  DveMotifRecipientListChangeChangedArg changed;
  DveMotifRecipientListChangeActionArg action;
} DveMotifRecipientListChangeArg;

typedef struct {
  XmListCallbackStruct* xcbs;
  DveMotifRecipientListChangeTag tag;
  DveMotifRecipientListChangeArg arg;
} DveMotifRecipientListCallbackStruct;

typedef void (*DveMotifRecipientListCallback)(DveMotifRecipientList list,
					      DveMotifRecipientListCallbackStruct* cbs);

void dveMotifRecipientListSetCallback(DveMotifRecipientList list,
				      DveMotifRecipientListCallback callback);

/* As for dve{S,G}etUserData(), it is your responsibility to manage
 *  the memory pointed to by userData.
 */
void dveMotifRecipientListSetUserData(DveMotifRecipientList list,
				      void* userData);
void* dveMotifRecipientListGetUserData(DveMotifRecipientList list);

/* Gives access to the list of selected members.  Do *not* modify this
 *  list; to change it, use the [de]select functions below.
 */
const DvePartList*
  dveMotifRecipientListGetPartList(DveMotifRecipientList list);

/* Meaningful only in browse mode; in multiple-select mode, returns -1.
 */
int dveMotifRecipientListGetSelection(DveMotifRecipientList list);

/* The following do not invoke the callback */
void dveMotifRecipientListSelect(DveMotifRecipientList list,
				 DveUserHandle member);
void dveMotifRecipientListSelectThese(DveMotifRecipientList list,
				      DvePartList* members,
				      Boolean giveCallbacks);
void dveMotifRecipientListSelectAll(DveMotifRecipientList list);
void dveMotifRecipientListDeselect(DveMotifRecipientList list,
				   DveUserHandle member);
void dveMotifRecipientListDeselectThese(DveMotifRecipientList list,
					DvePartList* members,
					Boolean giveCallbacks);
void dveMotifRecipientListDeselectAll(DveMotifRecipientList list);
void dveMotifRecipientListSetSelection(DveMotifRecipientList list,
				       DvePartList* members,
				       Boolean giveCallbacks);

/* Call this when an OpenDVE event changes the set of active
 * members.
 */
void dveMotifRecipientListHandleDveEvent(DveMotifRecipientList list,
					 DveEvent* event);

/* If autoSelect is set to True, members will be selected as soon as
 *  they accept (when the conference starts, all members will be
 *  selected).  The default is False.
 *
 * Clearly, autoSelect is False when the list is first created.  If
 *  it is set to True before any X or OpenDVE events are processed
 *  for the list (immediately after creation should do), all
 *  conference members will be selected.
 *
 * autoSelect is ignored for browse-mode lists.
 */
void dveMotifRecipientListSetAutoSelect(DveMotifRecipientList list,
					Boolean autoSelect);
Boolean dveMotifRecipientListGetAutoSelect(DveMotifRecipientList list);

/* Used to implement Cancel in a dialog; use Cache when the dialog
 *  is popped up, Restore when Cancel is pressed.  Restore invokes the
 *  callback for each entry [de]selected.
 */
void dveMotifRecipientListCacheSelections(DveMotifRecipientList list);
void dveMotifRecipientListRestoreSelections(DveMotifRecipientList list);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_MOTIF_RECIPIENT_LIST_H_*/
