/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the protocol for the C! 4.0 locator system.
 *
 * Requires:
 *
 * Includes:
 *  <IStypes.h>
 *  <dve/low.h>
 *
 * Implemented by:
 *  As-yet undetermined.
 *
 * Types:
 *
 * Macros:
 *
 * Author:
 *  francis, sokolsky
 *
 * Change Log:
 *  $Log$
 */

#ifndef _INSOFT_ENGINE_LOCATOR_PROTOCOL_H_
#define _INSOFT_ENGINE_LOCATOR_PROTOCOL_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#include <IStypes.h>
#include <dve/low.h>

#define LOC_UDP_PORT 6499

#define LOC_MAX_TTL 16

/* This is going to have to be declared elsewhere, actually, since it's
 * also used in ISnp2.h.
 */
#define SERIAL_NUMBER_LENGTH  64

/* flags for LocRequestPacket::flags, below */
typedef enum {
  locReqFlagNull = 0,
  locReqFlagTunneled = 1 /* tunneled: unicast, to be polycast by receiver */
} LocReqFlags;

typedef enum {
  locTagInvalid=0,
  locTagRequest,
  locTagReply
  } LocTag;

/*
 * This is the packet polycast (i.e., either broad- or multicast) onto
 * the net to locate engines.  All 16-bit entries are in network order.
 */
typedef struct {
  char userName[DVE_MAXNAME];          /* originator user name; can be "" */
  char hostName[DVE_MAXHOST];         /* originator host name */
  u_int8 ttl;         /* time to live */
  u_int8 hopsSoFar;   /* # of hops it's taken so far (including the originator)
		       * (# of entries in hops[]) */
  u_int16 flags;      /* options; the only one defined is tunneled */
  u_int16 senderPort; /* port # of originator, to which to reply (net order) */
  u_int16 maxTTL;     /* set equal to LOC_MAX_TTL; total size of hops[] */
  DveIPA hops[LOC_MAX_TTL];  /* addrs of hosts that sent this packet
			      * (originator first) */
  char topic[1];      /* string: topic of the conference; for filtering */
} LocRequestArg;

#ifdef __cplusplus
inline void fillLocRequestPacket(LocRequestPacket *p, char ser[], char user[], char host[])
{
  p->size = htonl(sizeof(LocRequestArg));
  p->tag = htonl(1);
  assert(lstrlen(ser) < SERIAL_NUMBER_LENGTH);
  lstrcpy(p->serial, ser);
  assert(lstrlen(user) < DVE_MAXNAME);
  lstrcpy(p->userName, user);
  assert(lstrlen(host) < DVE_MAXHOST);
  lstrcpy(p->hostName, host);
  p->ttl = 1;
  p->hopsSoFar = 1;
  p->flags = htons(1);
  p->reqID = htonl(0); /* ?? Sasha, I suggest assigning it here--francis */
  p->senderPort = htons(LOC_UDP_PORT);
  p->maxTTL = htons(LOC_MAX_TTL);
  p->hops[0] = 
  p->topic[0] = '\0';
}
#endif /* _cplusplus */

typedef enum {
  locConfTagInvalid=0,
  locConfTagInConf,
  locConfTagNotInConf,
  locConfTagUnknown
} LocConfTag;

typedef struct {
  u_int16 inConference,align;
  char userName[DVE_MAXNAME]; /* user & host name of user found */
  char hostName[DVE_MAXHOST]; /* May or may not be fully qualified; may not be
			       * partially qualified. */
  DveIPAList addrs; /* Note that the conference engine should be given the
		     * address, not the host name, when initiating; it may
		     * not be able to resolve the host name.
		     */
} LocReplyArg;

typedef union {
  LocRequestArg request;
  LocReplyArg reply;
} LocPacketArg;

/* reqID is not in network order, since it's meaningful only to the
 * original sender, and just gets copied over.
 */
typedef struct {
  u_int32 size;       /* total size of the packet */
  u_int32 tag;        /* Type of message, actually LocTag */
  u_int32 reqID;      /* same as ID of the request we're replying to */
  char serial[SERIAL_NUMBER_LENGTH];  /* Hashed serial # of replier; PC only */
  LocPacketArg arg;
} LocPacket;

#ifdef __cplusplus
}
#endif

#endif /*_INSOFT_ENGINE_LOCATOR_PROTOCOL_H_*/
