/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the OpenDVE API.
 *
 * Requires:
 *  None
 *
 * Includes:
 *  <netinet/in.h>
 *  <IStypes.h>
 *  <dve/low.h>
 *  <dve/events.h>
 *  <dve/errs.h>
 *  <dve/mem.h>
 *
 * Implemented by:
 *  dve.c
 *
 * Types:
 *  DveSession
 *  DveUserCallback
 *  DveUserCallbackNode
 *  DveEventCallback
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
//Required
 * Author:
 *
 * Designer:
 *
//Required, if applicable
 * Known Bugs:
 *
 * Notes:
 *
//Required
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_MAIN_API_H_
#define _DVE_MAIN_API_H_

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <netinet/in.h>

#include <IStypes.h>
#include <ISsys/select.h>
#include <dve/low.h>
#include <dve/events.h>
#include <dve/errs.h>
#include <dve/mem.h>

/* To have the system pick a port # for you, bind to DveBindAnyPort */
enum {DveBindAnyPort=0};

typedef struct _DveSession* DveSession;

typedef struct {
  DvePort destPort, srcPort;
  u_int32 routingFlags;
  u_int32 pluginProtocol; /* Which plugin's protocol is being used?
			   * Same as plugin's port # */
  u_int32 tag; /* Message tag, for private use by the plugin. */
  DvePartList destParts; /* Participants to receive the message. */
  /*void* windows;*/ /*Used in MS Windows version of API*/
} DveDataHeader;

typedef void (*DveUserCallback)(DveSession dveSession,
				DveUserHandle sender,
				DveDataPacket* packet);
/* Callback function when an event comes in. */
typedef void (*DveEventCallback)(DveSession dveSession, DveEvent* event);

DveSession dveOpenSession(DvePort port);
void dveCloseSession(DveSession dveSession);

/* Used when fork()ing off a child that is not going to do an exec();
 *  frees up the session's resources & closes the pipe.  This is needed
 *  to make sure that, if the parent dies, the child will not keep the
 *  connection open & confuse the engine.  It's not needed if you do
 *  an exec(), since the pipe's close-on-exec flag is set.  (If you
 *  used dveCloseSession() instead, the engine would get a goodbye
 *  message and close the connection to the parent.)
 */
void dveDetachSession(DveSession dveSession);

/* A destructor is a callback to be invoked when the session is closed
 *  or detached.  Destructors are invoked in the reverse order from
 *  that in which they were added.
 */
typedef void (*DveDestructor)(DveSession dveSession,
			      void* context);
typedef struct _DveDestructorId* DveDestructorId;
DveDestructorId dveAddDestructor(DveSession dveSession,
				 DveDestructor destructor,
				 void* context);
void dveDelDestructor(DveDestructorId destructor);

/* Initialize the session to use the given port# */
/*int dveInitSession(DveSession dveSession, DvePort port);*/
/* Connect the session to the conference engine */
/*int dveConnect(DveSession dveSession);*/
/* Disconnect the session from the conference engine */
/*int dveClose(DveSession dveSession);*/

/* Get the port on which the session is bound--useful if you bound
 *  to an arbitrary port (port DveBindAnyPort), or if you have multiple
 *  sessions.
 */
DvePort dveGetPort(DveSession dveSession);

/* Set/get the callback for the given event.  Unlike an Xt callback, an
 * OpenDVE callback is set, not added; a session may have only one callback
 * per event type or plugin protocol.
 */
int dveSetEventCallback(DveSession dveSession,
			DveEventTag event,
			DveEventCallback callback);
DveEventCallback dveGetEventCallback(DveSession dveSession,
				     DveEventTag event);

typedef struct _DveEventCallbackId* DveEventCallbackId;
typedef void (*DveAddedEventCallback)(DveSession dveSession,
				      DveEvent* event,
				      DveEventCallbackId id,
				      void* context);
DveEventCallbackId dveAddEventCallback(DveSession dveSession,
				       DveEventTag event,
				       DveAddedEventCallback callback,
				       void* context);
void dveDelEventCallback(DveEventCallbackId id);

/* Set/get the user-message callback for the given plugin protocol. */
int dveSetUserCallback(DveSession dveSession,
		       u_int32 pluginProtocol,
		       DveUserCallback callback);
DveUserCallback dveGetUserCallback(DveSession dveSession,
				   u_int32 pluginProtocol);

typedef struct _DveUserCallbackId* DveUserCallbackId;
typedef void (*DveAddedUserCallback)(DveSession dveSession,
				     DveUserHandle sender,
				     DveDataPacket* packet,
				     DveUserCallbackId id,
				     void* context);
DveUserCallbackId dveAddUserCallback(DveSession dveSession,
				     u_int32 pluginProtocol,
				     DveAddedUserCallback callback,
				     void* context);
void dveDelUserCallback(DveUserCallbackId id);

/* Set/get the event mask for the session.  Can be used to block previously
 * set callbacks.  Setting a callback to non-null unmasks that event type;
 * setting the callback to null masks it.
 */
/*
DveEventMask dveSetEventMask(DveSession dveSession,
			     DveEventMask mask);
DveEventMask dveGetEventMask(DveSession dveSession);
*/

/* Receive/don't receive the given event.  Returns <0 on failure. */
/*
int dveReceiveEvent(DveSession dveSession, DveEventTag tag);
int dveMaskOutEvent(DveSession dveSession, DveEventTag tag);
*/

/* Set/get the application-specific data of the session.  Useful when your
 * app has multiple OpenDVE sessions.
 */
int dveSetUserData(DveSession dveSession, void* userData);
void* dveGetUserData(DveSession dveSession);

/* tags beginning with '_' are reserved for internal use. */
int dveSetTaggedUserData(DveSession dveSession,
			 const char* tag,
			 void* userData);
void* dveGetTaggedUserData(DveSession dveSession,
			   const char* tag);

/* Get member info for this member/the specified member/all members */
int dveGetLocalMember(DveSession dveSession, DveConfParticipant* member);
int dveGetLocalMemberLong(DveSession dveSession,
			  DveConfParticipantLong* member);
int dveGetOneMember(DveSession dveSession,
		    DveUserHandle handle,
		    DveConfParticipant* member);
int dveGetAllMembers(DveSession dveSession,
		     DveConfParticipant* members, int maxMembers);
Bool16 dveIsMemberActive(DveSession dveSession, DveUserHandle handle);
int dveGetAllMembersWithActive(DveSession dveSession,
			       DveEventStartingArg_Member* members,
			       int maxMembers);

/* Shorthand stuff */
DveUserHandle dveGetLocalUserHandle(DveSession dveSession);
const char* dveGetLocalUserName(DveSession dveSession);
const char* dveGetLocalHostName(DveSession dveSession);
const char* dveGetLocalShortHostName(DveSession dveSession);
const char* dveShortenHostName(const char* hostName);
Bool16 dveInConference(DveSession dveSession);
Bool16 dveThisIsLocalUser(DveUserHandle handle);

/* Fill out the data packet, with sensible defaults:
 *  size is filled out correctly for the given data length
 *  srcPort is set to the port specified for the session (this cannot
 *   be spoofed in any event; it is always set to the port of the sender)
 *  destPort is set to the same as srcPort, since that's the normal case
 *  pluginPortocol is set to the port # of the session
 *  routingFlags is set to null (ordinary behavior)
 *  destParts is set to null
 */
void dveInitDataPacket(DveSession dveSession,
		       DveDataPacket* packet,
		       u_int32 datalen);
/* Allocate a data packet for the given data length, initialized with
 *  dveInitDataPacket(), above.  Returns NULL on failure. */
DveDataPacket* dveMallocDataPacket(DveSession dveSession,
				   int datalen);
/* Free a data packet allocated with dveMallocDataPacket().  Do not use
 *  any other free function for this purpose.
 */
void dveFreeDataPacket(DveDataPacket* packet);

/* Send a data packet from the given session. */
int dveSendDataPacket(DveSession dveSession, DveDataPacket* packet);
int dveSendMsg(DveSession dveSession,
	       DveDataHeader* header,
	       void* data,
	       u_int32 datalen);

/* Sends a synchronization packet to the engine & blocks until it gets a
 *  reply.  If any events or plugin messages come in in the meantime, they
 *  will be processed.  This lets you send some plugin messages & then do a
 *  sync to insure that they got processed by the engine before you quit
 *  (since, sometimes, closing down will discard any messages that have been
 *  queued).
 */
int dveSync(DveSession dveSession);

/* Functions to ping (test for the presence of) plugins.  "Blocks" means
 *  the same as for dveSync(); i.e., any events & plugin messages will get
 *  processed when they come in, rather than getting queued or something.
 *  It is not possible to ping yourself; dvePing() will report dveErrBadPort,
 *  and the other functions will leave you out of their count of expected
 *  replies.  This is partly because it's a little silly to test to see
 *  if you're running, and partly because ping is a test of network
 *  reachability, and you cannot send a plugin message to yourself (see
 *  dveSendDataPacket()).
 */

/* Ping the given plugin belonging to the given user; blocks.  Returns
 *  1 if the plugin is present, 0 if not, or -1 on error.  If you
 *  pass 0 for the handle, it pings locally.
 */
int dvePing(DveSession dveSession,
	    DveUserHandle handle,
	    DvePort port);

typedef void (*DvePingReplyCallback)(DveSession dveSession,
				     DveUserHandle sender,
				     DveDataPacket* reply,
				     void* privateData);

/* Sends a ping to the given plugin belonging to all users; does not
 *  block.  Returns the # of possible responses.  As each response
 *  comes in, replyCallback will be invoked.  privateData is stored with
 *  each outstanding broadcast ping, and passed to replyCallback.
 */
int dvePingBroadcast(DveSession dveSession,
		     DvePort port,
		     DvePingReplyCallback replyCallback,
		     void* privateData);

/* Sends a broadcast ping, but blocks.  Fills out *found so that a
 *  conference member is in the set if and only if a plugin responded
 *  for that member; returns the # of plugins found.
 */
int dvePingScan(DveSession dveSession,
		DvePort port,
		DvePartList* found);

/* Callback invoked when you get a bounce message (in reply to a 
 *  plugin message with the dveRoutingFlagGiveBounce flag set).
 */
typedef void (*DveBounceCallback)(DveSession dveSession,
				  DveUserHandle handle,
				  DvePort port);
void dveSetBounceCallback(DveSession dveSession,
			  DveBounceCallback callback);
DveBounceCallback dveGetBounceCallback(DveSession dveSession);

/* Functions for non-X-based plugins. */
/* Enter the OpenDVE event loop; returns only when the session is closed. */
void dveLoop(DveSession dveSession);
/* Set readfds to contain the fds of the session, and only those fds */
int dveSetFds(DveSession dveSession, fd_set* readfds);
/* Add the fds of the session to readfds */
int dveAddFds(DveSession dveSession, fd_set* readfds);
/* Handle input arriving for the session */
void dveProcessIncoming(DveSession dveSession, fd_set* readfds);

/* Should the plugin start mapped? The argument checked for is
 * "-dveUnmapped"; if it's missing, returns True.  If it's
 * found, returns False and removes that argument from the array
 * (to make it easier to interpret the remaining arguments)
 * (note that you have to pass &argc instead of argc, so that it
 * can be decremented).
 */
Bool16 dveShouldStartMapped(int* argc, const char* argv[]);

/* Call this function to let the confmgr know you're ready to use. */
int dveReportPresence(DveSession dveSession);

typedef enum {
  dveGreekVersionRelease=0,
  dveGreekVersionAlpha='a',
  dveGreekVersionBeta='b',
  dveGreekVersionGamma='g'
} DveGreekVersion;

typedef struct {
  int major, minor, subminor;
  DveGreekVersion greek;
  int subgreek;
} DveLongVersion;

void dveGetAPIVersion(DveLongVersion* version);
const char* dveAPIVersionString(void);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_MAIN_API_H_*/
