/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the locator library routines for use by plugins.
 *
 * Includes:
 *  <netinet/in.h>
 *  <ISsys/select.h>
 *  <dve/_locator.h>
 *
 * Implemented by:
 *  src/lib/dveLocate/locate.c
 *
 * Types:
 *  DveLocator
 *  DveLocPoll
 *  DveLocReplyCallback
 *
 * Functions:
 *
 * Variables:
 *
 * Macros:
 *
//Required
 * Author:
 *
 * Designer:
 *
//Required, if applicable
 * Known Bugs:
 *
 * Notes:
 *
//Required
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_LOCATOR_API_H_
#define _DVE_LOCATOR_API_H_

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <netinet/in.h>
#include <ISsys/select.h>
#include <dve/_locator.h>

typedef struct _DveLocator* DveLocator;
typedef struct _DveLocPoll* DveLocPoll;

typedef void (*DveLocReplyCallback)(DveLocator locator,
				    DveLocPoll poll,
				    LocPacket* reply);

DveLocator dveLocOpen(void);
void dveLocClose(DveLocator locator);
int dveLocGetFds(DveLocator locator, int fds[], int maxfds);
void dveLocSetAppData(DveLocator locator, void* appData);
void* dveLocGetAppData(DveLocator locator);

/* Returns actual TTL (in case requested too great).  1 for LAN, 1 extra
 *  for each additional hop.
 */
u_int8 dveLocSetTTL(DveLocator locator, u_int8 ttl);
u_int8 dveLocGetTTL(DveLocator locator);

DveLocPoll dveLocPollNet(DveLocator locator, DveLocReplyCallback callback);
DveLocPoll dveLocPollHost(DveLocator locator,
			  DveLocReplyCallback callback,
			  const DveIPA* addr);
void dveLocClosePoll(DveLocPoll poll);
void dveLocSetPollAppData(DveLocPoll poll, void* appData);
void* dveLocGetPollAppData(DveLocPoll poll);

void dveLocProcessIncoming(DveLocator locator, fd_set* fds);

#ifdef __cplusplus
}
#endif

#endif /*_DVE_LOCATOR_API_H_*/
